package de.unihannover.orderingExperimentAnalysis;

import java.io.IOException;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.function.Function;

import de.unihannover.orderingExperimentAnalysis.SessionData.Defect;

public class AnalyzeExperimentLogs {

    public static void main(String[] args) throws Exception {
        try {
            analyzeLogs(getFiles());
        } catch (final Exception e) {
            System.out.println();
            e.printStackTrace(System.out);
        }
    }

    public static String[] getFiles() {
        return new String[] {
                "realDataAfter6.experiment.1499407298015.log",
                "experiment.1500046443565.log",
                "experiment.1500355238765.log",
                "logFromDump1.log",
                "logFromDump2.log",
                "experiment.1500588237926.log",
                "experiment.1500832869230.log",
                "experiment.1501265956722.log"
        };
    }

    private static void analyzeLogs(String... filenames) throws IOException {
        final LogRecords log = LogRecords.load(filenames);
        final Collection<SessionData> sessions = log.getSplitByParticipant();

        System.out.println("Number of sessions: " + sessions.size());

        final Map<String, Integer> countsPerState = getCountsPerLastState(sessions);
        for (final Entry<String, Integer> e : countsPerState.entrySet()) {
            System.out.println("  last state " + e.getKey() + ": " + e.getValue());
        }

        for (final SessionData s : sessions) {
            final Collection<String> unclassified = s.getUnclassifiedRemarks();
            if (!unclassified.isEmpty()) {
                System.out.println("== unclassified remarks " + s.getId());
                for (final String r : unclassified) {
                    System.out.println(r.replace("\"", "\\\"").replace(": #", ": putDef(ret, \"#") + "\", );");
                }
            }
        }

        sessions.removeIf((SessionData s) -> !(s.isRealEffort(0) || s.isRealEffort(1) || s.isRealEffort(2)));
        System.out.println("Number of sessions with at least one review completed: " + sessions.size());
        System.out.println("Number of sessions with all needed data completed: " + sessions.stream().filter(AnalyzeExperimentLogs::neededDataComplete).count());

        System.out.println();

        final Map<String, Function<SessionData, ? extends Object>> columns = new LinkedHashMap<>();
        columns.put("pId", (SessionData s) -> "p" + s.getId());
        columns.put("shortId", SessionData::getShortId);
        columns.put("totalDuration", SessionData::getTotalDuration);
        columns.put("excludeAsDuplicate", SessionData::isDuplicateParticipation);
        columns.put("role", (SessionData s) -> s.getVariable("role"));
        columns.put("profDevExp", (SessionData s) -> s.getVariable("profDevExp"));
        columns.put("javaExp", (SessionData s) -> s.getVariable("javaExp"));
        columns.put("programPract", (SessionData s) -> s.getVariable("programPract"));
        columns.put("reviewPract", (SessionData s) -> s.getVariable("reviewPract"));
        columns.put("workStartHours", (SessionData s) -> toNumber(s.getVariable("workStartHours")));
        columns.put("fitness", (SessionData s) -> s.getVariable("fitness"));
        columns.put("jedit", (SessionData s) -> s.getVariable("jedit", ""));
        columns.put("prize", SessionData::isPrize);
        columns.put("controlledSetting", SessionData::isControlledSetting);
        columns.put("treatmentCombination", SessionData::getTreatmentCombination);
        columns.put("wmspanScore", SessionData::getWMSpanScore);
        columns.put("wmspanMathErrors", SessionData::getWMSpanMathErrors);
        columns.put("wmspanGrossTime", SessionData::getWMSpanGrossTime);
        columns.put("viewWidth", SessionData::getViewWidth);
        columns.put("viewHeight", SessionData::getViewHeight);
        columns.put("screenWidth", SessionData::getScreenWidth);
        columns.put("screenHeight", SessionData::getScreenHeight);
        columns.put("browser", SessionData::getBrowser);
        columns.put("netTimeForReviewWithoutIdlingR0", (SessionData s) -> s.isRealEffort(0) ? s.getNetTimeForReview(0, false) : "NA");
        columns.put("netTimeForReviewWithIdlingR0", (SessionData s) -> s.isRealEffort(0) ? s.getNetTimeForReview(0, true) : "NA");
        columns.put("grossTimeForReviewR0", (SessionData s) -> s.getGrossTimeForReview(0));
        columns.put("defectsFoundR0", (SessionData s) -> s.isRealEffort(0) ? s.getDefectsFoundForReview(0) : "NA");
        columns.put("defectsFoundAfterTenMinR0", (SessionData s) -> s.isRealEffort(0) ? s.getDefectsFoundAfter(0, 10) : "NA");
        columns.put("defectsFoundAfterTwentyMinR0", (SessionData s) -> s.isRealEffort(0) ? s.getDefectsFoundAfter(0, 20) : "NA");
        columns.put("defectsFoundAfterThirtyMinR0", (SessionData s) -> s.isRealEffort(0) ? s.getDefectsFoundAfter(0, 30) : "NA");
        columns.put("defectsTotalR0", (SessionData s) -> s.getDefectsTotalForReview(0));
        columns.put("timeForUnderstandingR0", (SessionData s) -> s.getGrossTimeForUnderstanding(0));
        columns.put("understandingCorrectR0", (SessionData s) -> s.getUnderstandingCorrect(0));
        columns.put("understandingTotalR0", (SessionData s) -> s.getUnderstandingTotal(0));
        columns.put("scrollAmountR0", (SessionData s) -> s.getScrollAmount(0));
        columns.put("pauseCountR0", (SessionData s) -> s.getPauseCount(0));
        columns.put("descriptionCountR0", (SessionData s) -> s.getDescriptionCount(0));
        columns.put("treatmentR1", (SessionData s) -> s.getTreatment(1));
        columns.put("changeR1", (SessionData s) -> s.getChange(1));
        columns.put("netTimeForReviewWithoutIdlingR1", (SessionData s) -> s.isRealEffort(1) ? s.getNetTimeForReview(1, false) : "NA");
        columns.put("netTimeForReviewWithIdlingR1", (SessionData s) -> s.isRealEffort(1) ? s.getNetTimeForReview(1, true) : "NA");
        columns.put("grossTimeForReviewR1", (SessionData s) -> s.getGrossTimeForReview(1));
        columns.put("defectsFoundR1", (SessionData s) -> s.isRealEffort(1) ? s.getDefectsFoundForReview(1) : "NA");
        columns.put("defectsFoundAfterTenMinR1", (SessionData s) -> s.isRealEffort(1) ? s.getDefectsFoundAfter(1, 10) : "NA");
        columns.put("defectsFoundAfterTwentyMinR1", (SessionData s) -> s.isRealEffort(1) ? s.getDefectsFoundAfter(1, 20) : "NA");
        columns.put("defectsFoundAfterThirtyMinR1", (SessionData s) -> s.isRealEffort(1) ? s.getDefectsFoundAfter(1, 30) : "NA");
        columns.put("defectsTotalR1", (SessionData s) -> s.isRealEffort(1) ? s.getDefectsTotalForReview(1) : "NA");
        columns.put("timeForUnderstandingR1", (SessionData s) -> s.getGrossTimeForUnderstanding(1));
        columns.put("understandingCorrectR1", (SessionData s) -> s.getUnderstandingCorrect(1));
        columns.put("understandingTotalR1", (SessionData s) -> s.getUnderstandingTotal(1));
        columns.put("scrollAmountR1", (SessionData s) -> s.getScrollAmount(1));
        columns.put("pauseCountR1", (SessionData s) -> s.getPauseCount(1));
        columns.put("descriptionCountR1", (SessionData s) -> s.getDescriptionCount(1));
        columns.put("treatmentR2", (SessionData s) -> s.getTreatment(2));
        columns.put("changeR2", (SessionData s) -> s.getChange(2));
        columns.put("netTimeForReviewWithoutIdlingR2", (SessionData s) -> s.isRealEffort(2) ? s.getNetTimeForReview(2, false) : "NA");
        columns.put("netTimeForReviewWithIdlingR2", (SessionData s) -> s.isRealEffort(2) ? s.getNetTimeForReview(2, true) : "NA");
        columns.put("grossTimeForReviewR2", (SessionData s) -> s.getGrossTimeForReview(2));
        columns.put("defectsFoundR2", (SessionData s) -> s.isRealEffort(2) ? s.getDefectsFoundForReview(2) : "NA");
        columns.put("defectsFoundAfterTenMinR2", (SessionData s) -> s.isRealEffort(2) ? s.getDefectsFoundAfter(2, 10) : "NA");
        columns.put("defectsFoundAfterTwentyMinR2", (SessionData s) -> s.isRealEffort(2) ? s.getDefectsFoundAfter(2, 20) : "NA");
        columns.put("defectsFoundAfterThirtyMinR2", (SessionData s) -> s.isRealEffort(2) ? s.getDefectsFoundAfter(2, 30) : "NA");
        columns.put("defectsTotalR2", (SessionData s) -> s.isRealEffort(2) ? s.getDefectsTotalForReview(2) : "NA");
        columns.put("timeForUnderstandingR2", (SessionData s) -> s.getGrossTimeForUnderstanding(2));
        columns.put("understandingCorrectR2", (SessionData s) -> s.getUnderstandingCorrect(2));
        columns.put("understandingTotalR2", (SessionData s) -> s.getUnderstandingTotal(2));
        columns.put("scrollAmountR2", (SessionData s) -> s.getScrollAmount(2));
        columns.put("pauseCountR2", (SessionData s) -> s.getPauseCount(2));
        columns.put("descriptionCountR2", (SessionData s) -> s.getDescriptionCount(2));
        columns.put("comparisonUnderstanding", SessionData::interpretComparisonUnderstanding);
        columns.put("comparisonComplicated", SessionData::interpretComparisonComplicated);
        columns.put("pageReloads", SessionData::determinePageReloads);
        columns.put("q_W_A", SessionData::questionCorrectWA);
        columns.put("q_W_B", SessionData::questionCorrectWB);
        columns.put("q_C1_A", SessionData::questionCorrectC1A);
        columns.put("q_C1_B", SessionData::questionCorrectC1B);
        columns.put("q_C1_C", SessionData::questionCorrectC1C);
        columns.put("q_C1_D", SessionData::questionCorrectC1D);
        columns.put("q_C2_A", SessionData::questionCorrectC2A);
        columns.put("q_C2_B", SessionData::questionCorrectC2B);
        columns.put("q_C2_C", SessionData::questionCorrectC2C);
        for (final Defect d : WU.values()) {
            columns.put("d_WU_" + d, (SessionData s) -> s.foundDefect(0, d));
        }
        for (final Defect d : P1.values()) {
            columns.put("d_D1_" + d, (SessionData s) -> s.foundDefect(s.getReviewNumberWith(Patch.P1), d));
        }
        for (final Defect d : P2.values()) {
            columns.put("d_D2_" + d, (SessionData s) -> s.foundDefect(s.getReviewNumberWith(Patch.P2), d));
        }
        for (final Defect d : WU.values()) {
            columns.put("timeUntil_WU_" + d, (SessionData s) -> s.getNetTimeUntilFound(0, d));
        }
        for (final Defect d : P1.values()) {
            columns.put("timeUntil_D1_" + d, (SessionData s) -> s.getNetTimeUntilFound(s.getReviewNumberWith(Patch.P1), d));
        }
        for (final Defect d : P2.values()) {
            columns.put("timeUntil_D2_" + d, (SessionData s) -> s.getNetTimeUntilFound(s.getReviewNumberWith(Patch.P2), d));
        }

        //print header
        for (final String s : columns.keySet()) {
            System.out.print(s);
            System.out.print(";");
        }
        System.out.println();

        //print rows
        for (final SessionData s : sessions) {
            for (final Function<SessionData, ? extends Object> f : columns.values()) {
                final Object o = f.apply(s);
                System.out.print(o == null ? "NA" : o.toString());
                System.out.print(";");
            }
            System.out.println();
        }
    }

    private static Map<String, Integer> getCountsPerLastState(Collection<SessionData> sessions) {
        final Map<String, Integer> ret = new LinkedHashMap<>();
        for (final SessionData s : sessions) {
            final String lastState = s.getLastState();
            final Integer oldCount = ret.get(lastState);
            ret.put(lastState, oldCount == null ? 1 : oldCount + 1);
        }
        return ret;
    }

    static double toNumber(String variable) {
        return Double.parseDouble(variable.replace(',', '.'));
    }

    private static boolean neededDataComplete(SessionData s) {
        return !s.isJeditIncomplete() && s.isRealEffort(0) && s.isRealEffort(1) && s.isRealEffort(2);
    }

}
