package de.unihannover.orderingExperimentAnalysis;

import java.io.IOException;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

public class CheckForDuplicateParticipation {

    public static void main(String[] args) throws Exception {
        try {
            checkDuplicateParticipation(AnalyzeExperimentLogs.getFiles());
        } catch (final Exception e) {
            System.out.println();
            e.printStackTrace(System.out);
        }
    }

    private static void checkDuplicateParticipation(String... filenames) throws IOException {
        final LogRecords log = LogRecords.load(filenames);
        final Collection<SessionData> sessions = log.getSplitByParticipant();

        //check for duplicate IPs
        final Multimap<String, SessionData> perIp = new Multimap<>();
        for (final SessionData s : sessions) {
            final String lastState = s.getLastState();
            if (!(lastState.equals("fresh") || lastState.equals("started"))) {
                perIp.put(s.getExternalIpAtStart(), s);
            }
        }

        for (final Entry<String, List<SessionData>> e : perIp.getEntries()) {
            if (e.getValue().size() > 1) {
                System.out.println("== IP: " + e.getKey());
                for (final SessionData session : e.getValue()) {
                    System.out.println("  " + session.getId() + ", " + session.getLastState()
                        + ", " + session.getInternalIp() + ", " + session.getBrowser());
                }
            }
        }
        System.out.println();

        //check using cookies
        final Map<String, String> previousIds = new LinkedHashMap<>();
        for (final SessionData s : sessions) {
            final String prev = s.getPreviousSessionR1FromCookie();
            if (prev.equals("NA")) {
                previousIds.put(s.getId(), s.getId());
            } else {
                previousIds.put(s.getId(), prev);
            }
        }

        boolean change;
        do {
            change = false;
            for (final Entry<String, String> e : previousIds.entrySet()) {
                if (!e.getKey().equals(e.getValue())) {
                    final String replacement = previousIds.get(e.getValue());
                    if (!replacement.equals(e.getValue())) {
                        e.setValue(replacement);
                        change = true;
                    }
                }
            }
        } while (change);

        final Multimap<String, SessionData> perInitialSession = new Multimap<>();
        for (final SessionData s : sessions) {
            perInitialSession.put(previousIds.get(s.getId()), s);
        }
        for (final Entry<String, List<SessionData>> e : perInitialSession.getEntries()) {
            if (e.getValue().size() > 1) {
                System.out.println("== Initial session: " + e.getKey());
                for (final SessionData session : e.getValue()) {
                    System.out.println("  " + session.getId() + ", " + session.getLastState()
                        + ", " + session.getPreviousSessionR1FromCookie());
                }
            }
        }
        System.out.println();

    }
}
