package de.unihannover.orderingExperimentAnalysis;

import static de.unihannover.orderingExperimentAnalysis.SessionData.Defect.putDef;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import de.unihannover.orderingExperimentAnalysis.SessionData.Defect;
import de.unihannover.orderingExperimentAnalysis.SessionData.FP;

public enum P1 implements Defect {
    /**
     * Initialize path2 wrongly in RenameBrowserTask constructor (hunk j).
     */
    WRONG_PATH_ASSIGNMENT,
    /**
     * Swap from and to in call RenameBrowserTask constructor in VFSBrowser.rename(from) (hunk b).
     */
    FROM_TO_SWITCHED_RENAME1,
    /**
     * Swap from and to in call RenameBrowserTask constructor in VFSBrowser.rename(from,to) (hunk c).
     */
    FROM_TO_SWITCHED_RENAME2,
    /**
     * Missing latch.countDown when session == null in VFSBrowser.delete (hunk a).
     */
    MISSING_LATCH_DECREMENT,
    /**
     * Typo in Message (pat instead of path) (hunk n).
     */
    TYPO_PATH,
    /**
     * Wrong type in toString of one of the tasks (hunk i).
     */
    WRONG_TOSTRING_MKDIR,
    /**
     * InterruptedException is logged as Error in VFSBrowser.delete (hunk a).
     */
    INTERRUPTED_EXCEPTION,
    /**
     * Provide wrong boolean value to setCancellable in one of the tasks (hunk h).
     */
    CANCELLABLE_WRONG,
    /**
     * Missing initialization of this.loadInfo in ListDirectoryBrowserTask constructor (hunk k).
     */
    LOADINFO_NOT_SET;

    @Override
    public String getShortString() {
        return this.getClass().getSimpleName() + "_" + this.ordinal();
    }

    @Override
    public boolean isDelocalized() {
        return this == FROM_TO_SWITCHED_RENAME1 || this == FROM_TO_SWITCHED_RENAME2;
    }

    static Map<String, List<Defect>> getTrueDefects() {
        final Map<String, List<Defect>> ret = new LinkedHashMap<>();
        putDef(ret, "#aR;659;Not Relavent Javadoc", FP.FP); //documentation
        putDef(ret, "#aR;659;description does not match with method", FP.FP); //documentation
        putDef(ret, "#aR;663;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;669;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;672;add else into previous line to avoid one unwated", FP.FP); //code style
        putDef(ret, "#aR;672;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;673;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;680;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;693;return must have a variable", FP.FP); //code style
        putDef(ret, "#aR;698;return must have a variable", FP.FP); //code style
        putDef(ret, "#aR;700;never used again?", FP.FP); //it is used again
        putDef(ret, "#aR;700;Use ThreadUtilities instead", FP.FP); //design issue
        putDef(ret, "#aR;702;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;706;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;707;this will lead to a hanging system as the await() call below will wait forever (as there will be no latch-incrementing tasks for the files skipped here)", P1.MISSING_LATCH_DECREMENT);
        putDef(ret, "#aR;707;this will lead to a hanging system as the await() call below will wait forever (as there will be no latch-decrementing tasks for the files skipped here)", P1.MISSING_LATCH_DECREMENT);
        putDef(ret, "#aR;707;latch size may not be consistent with tasks count because of this \"continue\", this may cause freeze", P1.MISSING_LATCH_DECREMENT);
        putDef(ret, "#aR;707;latch must be decremented", P1.MISSING_LATCH_DECREMENT);
        putDef(ret, "#aR;707;should call latch.countDown here", P1.MISSING_LATCH_DECREMENT);
        putDef(ret, "#aR;707;Hier wird CountDownLatch#countDown übersprungen, die Latch wird also nie 0 erreichen. Außerdem wird die VFS-Session nicht geschlossen.", P1.MISSING_LATCH_DECREMENT);
        putDef(ret, "#aR;707;Hier wird CountDownLatch#countDown übersprungen, die Latch wird also nie 0 erreichen", P1.MISSING_LATCH_DECREMENT);
        putDef(ret, "#aR;710;hier würde ich mal in der Doku nachsehen, was da synchronisiert sein muss", FP.FP);
        putDef(ret, "#aR;713;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;716;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;717;omfg", FP.FP);
        putDef(ret, "#aR;717;N listeners per N tasks will result in O(N^2) complexity", FP.FP); //correct, but no a correctness defect
        putDef(ret, "#aR;719;ggf. mit nächster Zeile vertauschen", FP.FP); //current code seems OK, too
        putDef(ret, "#aR;720;suspicious, listener maniulation from notification processing", FP.FP); //design issue
        putDef(ret, "#aR;720;Bitte sicherstellen, dass das keine ConcurrentModificationException oder Deadlock in TaskManager bewirken kann", FP.FP);
        putDef(ret, "#aR;720;which this is this? the inner class' or the VFSBrowser?", FP.FP);
        putDef(ret, "#aR;724;check if task is null", FP.FP);
        putDef(ret, "#aR;728;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;729;Think about a time out. If one delete task do never return you will wait forever.", FP.FP); //not a correctness defect
        putDef(ret, "#aR;729;timeout missing", FP.FP); //not a correctness defect
        putDef(ret, "#aR;729;this will block current thread which is AWT", FP.FP); //I don't think so
        putDef(ret, "#aR;729;please kill me", FP.FP); //well, no
        putDef(ret, "#aR;731;Wie kann das Warten auf die Latch interruptet werden? Sollten wir nur loggen oder es erneut versuchen?", P1.INTERRUPTED_EXCEPTION); //with some goodwill
        putDef(ret, "#aR;731;When this gets interrupted endRequest might get called before all tasks are done", P1.INTERRUPTED_EXCEPTION);
        putDef(ret, "#aR;732;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;733;Interrupted exception nicht vernünftig behandelt", P1.INTERRUPTED_EXCEPTION);
        putDef(ret, "#aR;733;missing call to Thread.interupted() after catching interrupted exception", P1.INTERRUPTED_EXCEPTION);
        putDef(ret, "#aR;733;InterruptedExceptions should not be swallowed.", P1.INTERRUPTED_EXCEPTION);
        putDef(ret, "#aR;733;the VFS manager error handler should handle the error.", FP.FP); //design issue
        putDef(ret, "#aR;733;Log.log should only have three parameters", FP.FP); //would be compile error
        putDef(ret, "#aR;733;handle InterruptedException correctly by calling thread.currentThread.interrupt", P1.INTERRUPTED_EXCEPTION);
        putDef(ret, "#aR;733;Why you implemented this differently from the other tasks?", FP.FP); //because it is bulk action running in parallel
        putDef(ret, "#aR;736;we are in AWT here, we can just call endRequest directly", FP.FP); //I think not true, otherwise design issue
        putDef(ret, "#aR;737;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;740;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#aR;741;endRequest muss in der EventQueue laufen, startRequest nicht? Kommt mir komisch vor", FP.FP);
        putDef(ret, "#bR;764;return must have a variable", FP.FP); //code style
        putDef(ret, "#bR;766;delayed", FP.FP); //variable naming
        putDef(ret, "#bR;766;funny name, but no. Also add some documentation.", FP.FP); //variable naming
        putDef(ret, "#bR;766;typo: should be \"delayed\"", FP.FP); //variable naming
        putDef(ret, "#bR;766;delayed? also, deleted //Do not change this line unltil ... is not actually satisfied, see VfsBrowser # delete", FP.FP);
        putDef(ret, "#bR;766;deleted //Do not change this line unltil ... is not actually satisfied, see VfsBrowser # delete", FP.FP);
        putDef(ret, "#bR;766;Typo: delated", FP.FP); //variable naming
        putDef(ret, "#bR;766;typo: delated -> delayed", FP.FP); //variable naming
        putDef(ret, "#bR;766;funny name, but no", FP.FP); //variable naming
        putDef(ret, "#bR;766;typo: delated", FP.FP); //variable naming
        putDef(ret, "#bR;766;deleted //Do not change this line unltil ... is not actually satisfied, see VfsBrowser # rename", FP.FP); //documentation
        putDef(ret, "#bR;766;delated?", FP.FP);
        putDef(ret, "#bR;766;typo?", FP.FP);
        putDef(ret, "#bR;766;do you mean \"delayedAWT\"?", FP.FP); //variable naming
        putDef(ret, "#bR;766;delated should be delayed", FP.FP); //variable naming
        putDef(ret, "#bR;767;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#bR;770;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#bR;771;wait for task end", FP.FP); //not needed, that's what the runnable is for
        putDef(ret, "#bR;775;this must be \"..., from, to, ...\"", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;from/to swapped?", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;the to and from should be re-ordered", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;to and from are swapped", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;switch from to", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;Same as above, to and from are swapped", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;wrong order of \"from\" and \"to\"", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;to/from sind vertauscht", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;from to parameter in wrong order", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;wrong param order (from/to)", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;to <-> from", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;to and from are in  the wrong order", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;to, from are in reverse order", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;from und to vertauscht", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;to and from swapped?", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;reverted to and from", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;check from, to", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;to from vertauscht?", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;Same as above, to and from are swapped. You also wrote delated", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;From and to have to be swapped", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;Is to and from in the correct order for the RenameBrowserTask?", P1.FROM_TO_SWITCHED_RENAME1);
        putDef(ret, "#bR;775;Hier bin ich mir nicht sicher, ob die Reihenfolge der Threads weiterhin korrekt läuft. Der AbstractBrowserTask hatte das awtRunnable ja auch früher, warum wurde dem das neue Runnable dann nicht schon früher übergeben...", FP.FP);
        putDef(ret, "#cR;798;At which time is this runnable executed? Is it save that the new directory has been created successfully before?", FP.FP);
        putDef(ret, "#cR;799;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#cR;779;No bug but i see this class and its old version twice.", FP.FP); //design issue
        putDef(ret, "#cR;779;No bug but i see this class and the method rename twice.", FP.FP); //design issue
        putDef(ret, "#cR;802;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#cR;803;wait for task end", FP.FP); //not needed, done by runnable
        putDef(ret, "#cR;806;consistent naming is less confusing. It's renameTask in other method", FP.FP);
        putDef(ret, "#cR;807;this must be \"..., from, to, ...\"", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;from/to should be swapped", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;from and to have to be swapped", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;to/from sind vertauscht", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;to, from wrong order?", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;Is to and from in the correct order for the RenameBrowserTask?", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;from und to vertauscht", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;order of from and to changed.", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;to, from -> from, to", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;806;to and from are in wrong order", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;wrong orgder of \"to\" and \"from\"", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;reverted to and from", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;806;Hier bin ich mir nicht sicher, ob die Reihenfolge der Threads weiterhin korrekt läuft. Der AbstractBrowserTask hatte das awtRunnable ja auch früher, warum wurde dem das neue Runnable dann nicht schon früher übergeben...", FP.FP);
        putDef(ret, "#cR;807;to from vertauscht?", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;Potentiell sind hier from und to vertauscht.", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;Parameter order from/to mixed up", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;You swapped the from and to parameters: in the old code you were using from before to", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#cR;807;Should be filename instead of from.", FP.FP); //no, from seems to be correct
        putDef(ret, "#cR;807;to/from seem to be in wrong order", P1.FROM_TO_SWITCHED_RENAME2);
        putDef(ret, "#dR;845;At which time is this runnable executed? Is it save that the new directory has been created successfully before?", FP.FP);
        putDef(ret, "#dR;845;Runs not in the same thread than before the code changes", FP.FP);
        putDef(ret, "#dR;845;deleted //Do not change this line unltil ... is not actually satisfied, see VfsBrowser # delete", FP.FP); //naming?
        putDef(ret, "#dR;845;deleted //Do not change this line unltil ... is not actually satisfied, see VfsBrowser # rename", FP.FP); //naming?
        putDef(ret, "#dR;846;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#dR;849;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#dR;850;wait for task end", FP.FP); //not needed, done by runnable
        putDef(ret, "#dR;851;can you access \"selected\" here or will you get a npe?", FP.FP); //yes you can, no problem to be seen
        putDef(ret, "#dR;853;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#dR;864;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#dR;872;Hier bin ich mir nicht sicher, ob die Reihenfolge der Threads weiterhin korrekt läuft. Der AbstractBrowserTask hatte das awtRunnable ja auch früher, warum wurde dem das neue Runnable dann nicht schon früher übergeben...", FP.FP);
        putDef(ret, "#eR;40;class is still needed", FP.FP); //no
        putDef(ret, "#eR;40;isn't a default constructor needed here? Afterall it is called from child references.", FP.FP);
        putDef(ret, "#fR;41;copy-pasted, be more specific", FP.FP); //documentation
        putDef(ret, "#fR;42;Missing JavaDoc for params \"session\", \"vfs\" and \"awtTask\"", FP.FP); //documentation
        putDef(ret, "#fR;43;The \"first\" in the comment is misleading. Shouldn't it be \"the path name to operate on\"?", FP.FP); //documentation
        putDef(ret, "#fR;43;incomplete", FP.FP); //documentation
        putDef(ret, "#fR;43;be specific, 'path to delete'?", FP.FP); //documentation
        putDef(ret, "#fR;43;comment should be adjusted - there is only one path", FP.FP); //documentation
        putDef(ret, "#fR;43;wrong JavaDoc: it should be a complete path", FP.FP); //documentation
        putDef(ret, "#fR;45;Maybe the class name is not properly chosen. It should tell something about always operating on a path", FP.FP); //design issue
        putDef(ret, "#fR;46;What about the Rename task, that has two paths instead of one?", FP.FP); //it has been moved to a subclass
        putDef(ret, "#fR;47;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#fR;53;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#gR;60;Make attributes private to prevent subclasses to access it directly. Make attributes final or provide access via setter to change the value.", FP.FP); //design issue
        putDef(ret, "#gR;61;Make attributes private to prevent subclasses to access it directly. Make attributes final or provide access via setter to change the value.", FP.FP); //design issue
        putDef(ret, "#gR;62;Make attributes private to prevent subclasses to access it directly. Make attributes final or provide access via setter to change the value.", FP.FP); //design issue
        putDef(ret, "#gR;63;Make attributes private to prevent subclasses to access it directly. Make attributes final or provide access via setter to change the value.", FP.FP); //design issue
        putDef(ret, "#gR;63;OK, here's my missing declaration of path", FP.FP); //just a remark
        putDef(ret, "#hR;10;javadoc missing", FP.FP); //documentation
        putDef(ret, "#hR;11;Was tut diese Klasse?", FP.FP); //documentation
        putDef(ret, "#hR;13;duplicate code", FP.FP);
        putDef(ret, "#hR;14;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#hR;15;several parameters are missing in JavaDoc", FP.FP); //documentation
        putDef(ret, "#hR;16;Nicht sinnvoller Kommentar", FP.FP); //documentation
        putDef(ret, "#hR;16;Kommentar sollte angepasst werden. Es handelt sich um Löschen", FP.FP);
        putDef(ret, "#hR;16;comment should be more concrete", FP.FP); //documentation
        putDef(ret, "#hR;18;Missing docs for session and vfs.", FP.FP); //documentation
        putDef(ret, "#hR;18;parameter description missing", FP.FP); //documentation
        putDef(ret, "#hR;19;\"the path to delete\"?", FP.FP); //documentation
        putDef(ret, "#hR;19;missing parameter details in javadoc", FP.FP); //documentation
        putDef(ret, "#hR;19;javadoc incomplete", FP.FP); //documentation
        putDef(ret, "#hR;19;The \"first\" in the comment is misleading. Shouldn't it be \"the path name to operate on\"?", FP.FP); //documentation
        putDef(ret, "#hR;19;describe all parameters", FP.FP); //documentation
        putDef(ret, "#hR;21;Missing Javadoc for params", FP.FP); //documentation
        putDef(ret, "#hR;22;this is useless", FP.FP);
        putDef(ret, "#hR;23;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#hR;24;Warum wird hier kein awtRunnable übergeben?", FP.FP); //is handled from the outside for delete
        putDef(ret, "#hR;24;why null instead of passing a runnable like in the other two commands?", FP.FP); //is handled from the outside for delete
        putDef(ret, "#hR;24;where's the awtRunnable?", FP.FP); //is handled from the outside for delete
        putDef(ret, "#hR;24;why null instead of passing a runnable?", FP.FP);
        putDef(ret, "#hR;29;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#hR;31;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#hR;32;muss true sein, war vorher cancellable", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;Warum ist dieser Task im Vergleich zu den anderen nicht abbrechbar? Fehler?", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;should be true", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;should setCancellable be set to true?", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;hier true? Sonst immer false", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;Warum wird cancellable hier auf false gesetzt? Ist true gemeint?", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;should be setCancellable(true)", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;passing same boolean value in like in catch block", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;Cancellable should be set to true at the start of the task.", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;old version was abortable, shold be cancelable?!", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;should probably set to true", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;Should it be setCancellable(true)?", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;32;should be true here", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;33;path is not declared any where", FP.FP);
        putDef(ret, "#hR;34;deleting or delete?", FP.FP); //property naming
        putDef(ret, "#hR;36;this.path", FP.FP); //code style
        putDef(ret, "#hR;36;session, path and browser is not declared any where", FP.FP);
        putDef(ret, "#hR;36;Should be a separate variable instead of changing the field value", FP.FP); //design issue
        putDef(ret, "#hR;36;same as above: could change path multiple times to different values, if _run is called more than once. Use separate local variable instead.", FP.FP); //design issue
        putDef(ret, "#hR;38;Unwanted empty space", FP.FP); //code style
        putDef(ret, "#hR;40;kein Log?", FP.FP);
        putDef(ret, "#hR;43;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#hR;44;Code Duplikat mit finally Block", FP.FP); //design issue
        putDef(ret, "#hR;44;line 44 until 47 is duplicated in 57 until 60.", FP.FP); //design issue
        putDef(ret, "#hR;44;line 44 until 47 is duplicated in 57 until 60. There is no return, so the finally block is useless.", FP.FP); //design issue
        putDef(ret, "#hR;44;Warum wird cancellable hier nochmal auf false gesetzt? Ist true gemeint?", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;45;nearly same code as in mkDir Task, should be reused, maybe an method template", FP.FP); //design issue
        putDef(ret, "#hR;46;pp ist kein sprechender Variablenname", FP.FP); //code style
        putDef(ret, "#hR;47;directory?", FP.FP); //seems to be OK
        putDef(ret, "#hR;47;directory->delete?", FP.FP); //seems to be OK
        putDef(ret, "#hR;47;ioerror.directory-error als static string machen und die anderen String auch", FP.FP); //code style
        putDef(ret, "#hR;47;error string inconsistent", FP.FP);
        putDef(ret, "#hR;47;In a case of an IOException, how do you want to distinguish the IOException in catch and finally clause?", FP.FP); //not a correctness defect
        putDef(ret, "#hR;47;ioerror.directory-error als static string machen", FP.FP);
        putDef(ret, "#hR;47;path and browser is not declared any where", FP.FP);
        putDef(ret, "#hR;47;Muss wahrscheinlich ioerror oder ioerror.delete-error statt ioerror.directory-error sein, aber aber vorher auch schon falsch durch Copy-Paste aus listDirectories() (?)", FP.FP);
        putDef(ret, "#hR;48;InterruptedException-Behandlung fehlt", FP.FP); //would be compile error?
        putDef(ret, "#hR;50;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#hR;52;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#hR;55;again, code duplication", FP.FP); // design issue
        putDef(ret, "#hR;56;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#hR;57;Code Duplikat mit catch Block des äußeren try, vereinheitlichen", FP.FP); //design issue
        putDef(ret, "#hR;57;same here: should be cancellable", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;57;Warum wird cancellable hier nochmal auf false gesetzt? Ist true gemeint?", P1.CANCELLABLE_WRONG);
        putDef(ret, "#hR;57;already set in like 44", FP.FP);
        putDef(ret, "#hR;57;Extract into method. Same as in line 42", FP.FP); //design issue
        putDef(ret, "#hR;59;pp ist kein sprechender Variablenname", FP.FP); //design issue
        putDef(ret, "#hR;60;directory?", FP.FP);
        putDef(ret, "#hR;60;directory->delete?", FP.FP);
        putDef(ret, "#hR;60;error string inconsistent", FP.FP);
        putDef(ret, "#hR;60;Muss wahrscheinlich ioerror oder ioerror.delete-error statt ioerror.directory-error sein, aber aber vorher auch schon falsch durch Copy-Paste aus listDirectories() (?)", FP.FP);
        putDef(ret, "#hR;60;path and browser is not declared any where", FP.FP); //no, it is
        putDef(ret, "#hR;60;In a case of an IOException, how do you want to distinguish the IOException in catch and finally clause?", FP.FP); //not a correctness defect
        putDef(ret, "#hR;60;See comment above: reuse this code", FP.FP); //design issue
        putDef(ret, "#hR;62;Assuming that setCancellable is equivalent to the original setAbortable a setCancellable might be missing in the no error case (after the finally) because it is present in listDirectory but not in the other replaced methods", FP.FP);
        putDef(ret, "#hR;62;Assuming that setCancellable is equivalent to the original setAbortable a setCancellable is missing in the no error case (after the finally)", FP.FP);
        putDef(ret, "#hR;65;Same code as in the file before. -> Extract", FP.FP); //design issue
        putDef(ret, "#hR;66;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#iR;1;This whole class looks like it has a lot of duplicates.", FP.FP); //design issue
        putDef(ret, "#iR;1;where are the tests. Difficult if not impossible to perform a code review for such a system without even a test...", FP.FP); //right, but no correctness defect
        putDef(ret, "#iR;11;Was tut diese Klasse?", FP.FP); //documentation
        putDef(ret, "#iR;11;Description missing", FP.FP); //documentation
        putDef(ret, "#iR;11;lol", FP.FP); //documentation (probably)
        putDef(ret, "#iR;12;* missing", FP.FP); //documentation
        putDef(ret, "#iR;13;no privilege attribute", FP.FP); //documentation
        putDef(ret, "#iR;14;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#iR;16;Nicht sinnvoller Kommentar", FP.FP); //documentation
        putDef(ret, "#iR;16;comment should be more concrete", FP.FP); //documentation
        putDef(ret, "#iR;16;Some of the parameters are missing in the JavaDoc", FP.FP); //documentation
        putDef(ret, "#iR;16;document, which request is created, ie. a mkdir request", FP.FP); //documentation
        putDef(ret, "#iR;16;Missing space between description and parameter", FP.FP); //documentation
        putDef(ret, "#iR;16;copy-pasted and wrong javadoc", FP.FP); //documentation
        putDef(ret, "#iR;17;other parameters are missing in description", FP.FP); //documentation
        putDef(ret, "#iR;18;please comment all parameter", FP.FP); //documentation
        putDef(ret, "#iR;18;Session, vfs and awtRunnable parameter missing from docs.", FP.FP); //documentation
        putDef(ret, "#iR;18;missing parameter details in javadoc", FP.FP); //documentation
        putDef(ret, "#iR;18;Bad parameter naming", FP.FP);
        putDef(ret, "#iR;18;The only path name to operate on", FP.FP); //documentation
        putDef(ret, "#iR;18;\"To operate on\", nicht \"to create\"?", FP.FP); //documentation
        putDef(ret, "#iR;18;The \"first\" in the comment is misleading. Shouldn't it be \"the path name to operate on\"?", FP.FP); //documentation
        putDef(ret, "#iR;18;missing parameters in javadoc", FP.FP); //documentation
        putDef(ret, "#iR;19;Parameter description for awtRunnable missing", FP.FP); //documentation
        putDef(ret, "#iR;20;There are parameters missing in JavaDoc description", FP.FP); //documentation
        putDef(ret, "#iR;20;Missing Javadoc for params", FP.FP); //documentation
        putDef(ret, "#iR;20;missing JavaDoc for params \"session\", \"vfs\" and \"awtRunnable\"", FP.FP); //documentation
        putDef(ret, "#iR;20;It is better to declare constructor visibility(public or private)", FP.FP); //why? code style
        putDef(ret, "#iR;22;constructor should not take awtRunnable", FP.FP);
        putDef(ret, "#iR;23;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#iR;24;awtRunnable should be null", FP.FP);
        putDef(ret, "#iR;25;This is useless", FP.FP); //don't know what is meant
        putDef(ret, "#iR;28;A lot of the code in the run Method is the same as in the other run methods. This code should be extracted into its own method", FP.FP); //design issue
        putDef(ret, "#iR;28;method name should not start with underscore", FP.FP);
        putDef(ret, "#iR;29;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#iR;30;args does not need to be an array", FP.FP);
        putDef(ret, "#iR;30;better use this.path", FP.FP);
        putDef(ret, "#iR;30;path is not declared any where", FP.FP);
        putDef(ret, "#iR;30;getPath()", FP.FP);
        putDef(ret, "#iR;30;should be getPath() ?? (don't know exact implementation of superclass)", FP.FP);
        putDef(ret, "#iR;32;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#iR;34;contents of args is changed later!", FP.FP);
        putDef(ret, "#iR;34;Whitespace after comma", FP.FP); //code style
        putDef(ret, "#iR;34;missing space after comma", FP.FP); //code style
        putDef(ret, "#iR;35;unnecessary line break", FP.FP); //code style
        putDef(ret, "#iR;36;style: spacing issue", FP.FP); //code style
        putDef(ret, "#iR;36;don;t change path field use separate canonPath consistently, see listFiles", FP.FP);
        putDef(ret, "#iR;36;session, path and browser is not declared any where", FP.FP);
        putDef(ret, "#iR;36;missing space after comma", FP.FP); //code style
        putDef(ret, "#iR;36;Assigned local value to class field", FP.FP); //?
        putDef(ret, "#iR;36;Whitespace after comma", FP.FP); //code style
        putDef(ret, "#iR;36;session and path is not declared any where", FP.FP);
        putDef(ret, "#iR;36;path is overriden here and used before as contents of array to determine a property", FP.FP);
        putDef(ret, "#iR;36;Changing the path variable is bad design. You do not know which path will be returned in toString.", FP.FP); //design issue
        putDef(ret, "#iR;36;don;t change path field use separate canonPath consistently, see ListDirectoryBrowserTask.java line 65", FP.FP); //design issue
        putDef(ret, "#iR;38;missing space after comma", FP.FP); //code style
        putDef(ret, "#iR;38;Whitespace after comma, curly braces", FP.FP); //code style
        putDef(ret, "#iR;38;session, path and browser is not declared any where", FP.FP);
        putDef(ret, "#iR;39;kein Logging hier?", FP.FP);
        putDef(ret, "#iR;39;Whitespace after comma", FP.FP); //code style
        putDef(ret, "#iR;39;spaces after commas", FP.FP); //code style
        putDef(ret, "#iR;39;path and browser is not declared any where", FP.FP);
        putDef(ret, "#iR;39;missing curly brackets {}", FP.FP); //code style
        putDef(ret, "#iR;39;null seems wrong, javadoc missing", FP.FP); //design issue, no exception available
        putDef(ret, "#iR;42;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#iR;43;Code Duplikt mit finally Block, vereinheitlichen.", FP.FP); //design issue
        putDef(ret, "#iR;43;line 43 to 46 and line 56 to 59 are the same: should be extracted into method", FP.FP); //design issue
        putDef(ret, "#iR;43;refactor extract method to avoid code repetition and rethink exception logic", FP.FP); //design issue
        putDef(ret, "#iR;43;in finally", FP.FP);
        putDef(ret, "#iR;43;consider moving into finally block to ensure it is called once", FP.FP); //I think this is OK because the task will end anyway when there is no exception
        putDef(ret, "#iR;44;Whitespace after comma", FP.FP);
        putDef(ret, "#iR;45;args contents is overriden again", FP.FP);
        putDef(ret, "#iR;45;better use a local variable", FP.FP);
        putDef(ret, "#iR;45;better use a local variable for args. use get io.getMessage() instead of io.toString() ?", FP.FP);
        putDef(ret, "#iR;45;possible nullpointer", FP.FP); //I see no nullpoiner there
        putDef(ret, "#iR;45;reassignment to path causes trouble (see toString method)", FP.FP);
        putDef(ret, "#iR;45;Kein Fehler, aber lieber neues String[] anlegen als ein unbeteiligtes Array zu recyclen", FP.FP); //design issue
        putDef(ret, "#iR;45;reusing of args is confusing, use separate new array here", FP.FP);
        putDef(ret, "#iR;45;toString method is not defined in IOException", FP.FP);
        putDef(ret, "#iR;46;In a case of an IOException, how do you want to distinguish the IOException in catch and finally clause?", FP.FP); //not a correctness defect
        putDef(ret, "#iR;46;error string inconsistent", FP.FP);
        putDef(ret, "#iR;46;Sollte das hier ioerror.mkdir-error sein, oder ist der (allgemeine?) ioerror ok?", FP.FP);
        putDef(ret, "#iR;46;space after comma", FP.FP);
        putDef(ret, "#iR;46;it would be better to log orifginal path before canonicalization", FP.FP);
        putDef(ret, "#iR;46;Whitespace after comma", FP.FP);
        putDef(ret, "#iR;46;string parameter should probably be replaced with \"ioerror.mkdir-error\"", FP.FP);
        putDef(ret, "#iR;46;same for this piece of code, you can create a function and reuse it as I commented above too", FP.FP); //design issue
        putDef(ret, "#iR;47;InterruptedException-Behandlung fehlt", FP.FP); //would be compile error?
        putDef(ret, "#iR;49;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#iR;50;do we teally need a nested try catch in the finally clause? Rethink exception logic", FP.FP); //design issue
        putDef(ret, "#iR;51;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#iR;52;Why is the start of the session not handled in this run method. What happens if you call run but forgot to start a session?", FP.FP); //design issue
        putDef(ret, "#iR;52;Whitespace after comma", FP.FP);
        putDef(ret, "#iR;54;This catch-Block looks exactly the same as the one above (line 41). Extract into a method to avoid duplicate code.", FP.FP);
        putDef(ret, "#iR;55;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#iR;56;Code Duplikat mit catch Block des äußeren try, vereinheitlichen.", FP.FP); //design issue
        putDef(ret, "#iR;56;muss in finally", FP.FP);
        putDef(ret, "#iR;56;this whole catch block is duplicated....refactor", FP.FP); //design issue
        putDef(ret, "#iR;56;line 43 to 46 and line 56 to 59 are the same: should be extracted into method", FP.FP); //design issue
        putDef(ret, "#iR;56;duplicate code; maybe catch the exception elsewhere?", FP.FP);
        putDef(ret, "#iR;56;copy-pasted block, most probably only Log.log is actually needed from fially block", FP.FP);
        putDef(ret, "#iR;56;copy-pasted block, most probably only Log.log is actually needed from finally block", FP.FP);
        putDef(ret, "#iR;57;copy-pasted block, correct the text to distinguish with line 46", FP.FP);
        putDef(ret, "#iR;57;Whitespace after comma", FP.FP);
        putDef(ret, "#iR;57;copy-pasted block, most probably only Log.log is actually needed from fially block", FP.FP);
        putDef(ret, "#iR;58;args contents is overriden again", FP.FP);
        putDef(ret, "#iR;58;duplicate code for error handling", FP.FP); //design issue
        putDef(ret, "#iR;58;copy-pasted block, remove this line at all, or at least use separate var instead of args, see comment for line 45", FP.FP);
        putDef(ret, "#iR;58;possible nullpointer", FP.FP);
        putDef(ret, "#iR;58;copy-pasted block, most probably only Log.log is actually needed from fially block", FP.FP);
        putDef(ret, "#iR;58;better use a local scoped variable", FP.FP);
        putDef(ret, "#iR;58;Kein Fehler, aber lieber neues String[] anlegen als ein unbeteiligtes Array zu recyclen", FP.FP); //Design issue
        putDef(ret, "#iR;58;better use a local scoped variable for args. use get io.getMessage() instead of io.toString() ?", FP.FP);
        putDef(ret, "#iR;59;In a case of an IOException, how do you want to distinguish the IOException in catch and finally clause?", FP.FP); //not a correctness defect
        putDef(ret, "#iR;58;copy-pasted block, remove this line at all", FP.FP);
        putDef(ret, "#iR;58;copy-pasted block, remove this line at all, or at least use separate var instead of args, see", FP.FP);
        putDef(ret, "#iR;59;string parameter should probably be replaced with \"ioerror.mkdir-error\"", FP.FP);
        putDef(ret, "#iR;59;copy-pasted block, most probably only Log.log is actually needed from fially block", FP.FP);
        putDef(ret, "#iR;59;copy-pasted block, remove this line or at least use different code instead of \"ioerror\"", FP.FP);
        putDef(ret, "#iR;59;path and browser is not declared any where", FP.FP);
        putDef(ret, "#iR;59;spaces after commas", FP.FP);
        putDef(ret, "#iR;59;Sollte das hier ioerror.mkdir-error sein, oder ist der (allgemeine?) ioerror ok?", FP.FP);
        putDef(ret, "#iR;59;Whitespace after comma", FP.FP);
        putDef(ret, "#iR;61;Assuming that setCancellable is equivalent to the original setAbortable a setCancellable might be missing in the no error case (after the finally) because it is present in listDirectory but not in the other replaced methods", FP.FP);
        putDef(ret, "#iR;62;missing setCancellable(false)? also consider moving to finally block", FP.FP); //I think this is OK because the task ends with the method
        putDef(ret, "#iR;63;override", FP.FP); //code style
        putDef(ret, "#iR;64;Missing @Override", FP.FP); //code style
        putDef(ret, "#iR;65;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#iR;66;better use string.format()", FP.FP); //code style
        putDef(ret, "#iR;66;type=DELETE does not match class name's suggested intention of creating dirs", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;MkDir or DELETE?", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;the message might be related to make directory not 'DELETE'", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;type = MKDIR?", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;Das ist kein DELETE sondern ein MKDIR oder?", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;type should be MKDIR and not DELETE?", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;this is \"mkdir\" task, not \"delete\"", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;Typo: Ist kein DELETE, sondern ein MKDIR", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;type should be MKDIR", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;type=MK_DIR?", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;DELETE is wrong type", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;That’s MKDIR, not DELETE", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;type seems to be wrong", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;Copy-paste-Fehler: Ist kein DELETE, sondern ein MKDIR", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;is DELETE the correct type?", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;type here should not be DELETE", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;ToString contains type=DELETE although the action is named MkDir", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;Type shoud be MKDIR or similar.", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;DELETE? It should probably be MKDIR or similar", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;wir sind nicht in DELETE", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;Is type=DELETE correct? Should it be MKDIR?", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;this not DEELTE but mkDir", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;Should bei MKDIR instead of DELETE", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;66;type should be CREATE and not DELETE", P1.WRONG_TOSTRING_MKDIR);
        putDef(ret, "#iR;67;Why do you use single quote for the last character?", FP.FP); //code style
        putDef(ret, "#iR;67;vfs and path not declared anywhere", FP.FP);
        putDef(ret, "#iR;67;Use \" instead of '", FP.FP); //code style
        putDef(ret, "#jR;13;Was tut diese Klasse?", FP.FP);
        putDef(ret, "#jR;13;javadoc missing", FP.FP); //documentation
        putDef(ret, "#jR;15;There is so much repetitin in all this Task. And little differences with no need. UNMANTAINABLE, Refactor", FP.FP); //design issue
        putDef(ret, "#jR;16;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;18;Nicht sinnvoller Kommentar", FP.FP); //documentation
        putDef(ret, "#jR;18;Comment should be more concrete", FP.FP); //documentation
        putDef(ret, "#jR;18;copy-pasted and wrong", FP.FP); //documentation
        putDef(ret, "#jR;21;Bad parameter naming", FP.FP);
        putDef(ret, "#jR;21;-> from", FP.FP);
        putDef(ret, "#jR;21;rename -> from", FP.FP); //documentation
        putDef(ret, "#jR;21;Kommentare unklar: Also ist das der bestehende Datei und der Ziel-Dateiname?", FP.FP);
        putDef(ret, "#jR;21;first/second... It is not clear which is from and which is to", FP.FP); //documentation
        putDef(ret, "#jR;21;incomplete", FP.FP); //documentation
        putDef(ret, "#jR;21;path1 better called \"from\" name", FP.FP);
        putDef(ret, "#jR;21;Session and vfs parameters are missing docs.", FP.FP);
        putDef(ret, "#jR;21;Session, vfs and awtRunnable parameters are missing docs.", FP.FP); //documentation
        putDef(ret, "#jR;21;Kommentare unklar: Also ist path1 die bestehende Datei und path2 der Ziel-Dateiname?", FP.FP);
        putDef(ret, "#jR;22;bad parameter naming", FP.FP);
        putDef(ret, "#jR;22;-> to?", FP.FP);
        putDef(ret, "#jR;22;rename -> to?", FP.FP);
        putDef(ret, "#jR;22;missing JavaDoc for params \"session\", \"vfs\", \"awtRunnable\"", FP.FP);
        putDef(ret, "#jR;22;path1 better called \"to\" name", FP.FP); //variable naming
        putDef(ret, "#jR;24;parameters missing in Javadoc description", FP.FP); //documentation
        putDef(ret, "#jR;24;Missing Javadoc for params", FP.FP); //documentation
        putDef(ret, "#jR;25;path1 und path2 in from und to umbenennen, um Lesbarkeit zu verbessern", FP.FP); //code style
        putDef(ret, "#jR;25;path2 were removed from its parent interface", FP.FP);
        putDef(ret, "#jR;25;parameter names are non self explaining and not documentated", FP.FP); //documentation
        putDef(ret, "#jR;27;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;28;Null checks can be added here", FP.FP);
        putDef(ret, "#jR;28;path1 were removed from its parent interface", FP.FP);
        putDef(ret, "#jR;28;path1 were removed from its parent class", FP.FP);
        putDef(ret, "#jR;29;this must be \"= path2\"", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;path2 should be assigned path2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;Hier muss this.path2, nicht this.path1 zugewiesen werden", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;I guess this should be this.path2 = path2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;should be set to paht2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;this.path2 = path2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;assigned wrong variable", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;the value in path2 is never used before and gets overriden here. Bug i think so ! Should be this.path2=path2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;should be this.path2 = path2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;Vermutlich sollte hier path2 übergeben werden?! Btw. Wo wird überhaupt das Attribut path2 deklariert?", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;wrong argument assignment?", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;What is the point of this? If this is intended -> Comment why, what is happening with the real parameter \"path2\" here?", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;wrong assignment, param path2 not used", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;= path2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;looks like a bad idea to have path1, path2 and this.path2 as different variable sin the same source unit", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;typo path1/path2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;probably meant too assign path2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;should be path2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;why this? shouldn't I set path2?", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;assigned wrong parameter", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;param path2 not used", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;path property from the super class is never assigned, but is used below", FP.FP);
        putDef(ret, "#jR;29;should be \"this.path2 = path2;\"", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;Assigning wrong value, should use path2.", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;this.path2 = path2 ?", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;the value in path2 is never used before and gets overriden here. Bug ? or variable not needed !", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;this.path2 = path2;", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;there is no path2 anymore....this should have been caught by the compiler", FP.FP);
        putDef(ret, "#jR;29;I think you would like to store the value of parameter path2 in the attribute path2. Otherwise you will never rename your file.", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;Isn't it path2 ?", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;What is the point of this? If this is intended -> Comment why", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;I think we should assign path2 to field path2 otherwise it would a little bit confusing", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;Assignment should be path2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;should be this.path2=path2", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;Assignment to wrong variable. How about naming these from/to or source/target?", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;this.path2 is set to value of path2, path2 is completely ignored", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;Es ist sicher path2 gemeint statt path1?", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;falsche zuweisung. muss path2 sein", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;Vermutlich sollte hier path2 übergeben werden?!", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;29;path2 = path1?", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;34;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;36;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;38;path1 statt path?", FP.FP);
        putDef(ret, "#jR;38;path is not declared", FP.FP);
        putDef(ret, "#jR;38;path2 were removed from its parent class", FP.FP);
        putDef(ret, "#jR;38;what is path? or do you mean path1", FP.FP); //is an attribute
        putDef(ret, "#jR;38;you mean path1. did you confuse this in the whole file? Or am I wrong?", FP.FP); //path1 is the constructor parameter, path is the attribute. is OK
        putDef(ret, "#jR;38;path should be path1", FP.FP); //path1 is the constructor parameter, path is the attribute. is OK
        putDef(ret, "#jR;38;use getPath()", FP.FP); //code style
        putDef(ret, "#jR;39;are you sure that it is status.renaming and not status.rename?", FP.FP); //no, I'm not, but there is nothing in the code that indicated "renaming" is wrong
        putDef(ret, "#jR;39;path property from the super class is never assigned, but is used here", FP.FP);
        putDef(ret, "#jR;41;same as above, path1", FP.FP); //no, the attribute name is path
        putDef(ret, "#jR;41;where is path declared?", FP.FP); //is an attribute
        putDef(ret, "#jR;41;use separate var, don't modify the super.xxx field", FP.FP); //design issue
        putDef(ret, "#jR;41;field value is overriden here. If vfs_canonPath does not produce the same result for an already canonicalized path this produces possibly unexpected behaviour", FP.FP); //design issue
        putDef(ret, "#jR;41;method parameters should not be assigned.", FP.FP); //design issue
        putDef(ret, "#jR;41;path is not declared", FP.FP);
        putDef(ret, "#jR;41;path1 instead of path only", FP.FP);
        putDef(ret, "#jR;41;Assigned local value to class field", FP.FP);
        putDef(ret, "#jR;41;use separate var, don't modify the super.xxx value", FP.FP);
        putDef(ret, "#jR;41;if _run() can be called more than once, path/path2 may be manipulated multiple times. Use different local variables here!", FP.FP); //design issue
        putDef(ret, "#jR;42;if _run() can be called more than once, path/path2 may be manipulated multiple times. Use different local variables here!", FP.FP); //design issue
        putDef(ret, "#jR;42;see previous line, path2 anyway is always path because it is not initialized wrongly in constructor", FP.FP);
        putDef(ret, "#jR;42;see previous line", FP.FP); //remark in previous line is "field value is overridden here ..."
        putDef(ret, "#jR;42;see previous line, path2 anyway is always null because it is not used in objects constructor", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;42;path2 is a bad identifier.. newPathName would be better", FP.FP); //naming
        putDef(ret, "#jR;42;use separate var, don't modify the field", FP.FP); //design issue
        putDef(ret, "#jR;42;path2 were removed from its parent interface", FP.FP);
        putDef(ret, "#jR;42;path and path2 will have the same value. See review mark in the constructor", P1.WRONG_PATH_ASSIGNMENT);
        putDef(ret, "#jR;44;at least rename to file2 (or better `toFile` or `destFile`)", FP.FP); //naming
        putDef(ret, "#jR;44;shouldn't  it be path instead of path2", FP.FP); //no, path is used in the statement above
        putDef(ret, "#jR;45;We should stop the renaming, if the does not exist.", FP.FP); //code is OK
        putDef(ret, "#jR;46;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;47;If the file system is case sensitive and the file exists you will always end with the error in the else clause.", FP.FP); //code is OK
        putDef(ret, "#jR;47;this should be if ((OperatingSystem.isCaseSensitiveFS()) && !path.equals(path2))", FP.FP); //code is OK
        putDef(ret, "#jR;47;If the rename target exists and the FS is case insensitive and the paths only differ in case (i.e. they don't differ at all the rename proceeds? Not sure what is supposed to be tested, but this sounds wrong", FP.FP);
        putDef(ret, "#jR;47;is the operation vfs._getFile(..)  case insensitive?", FP.FP); //depends on file system
        putDef(ret, "#jR;47;Too much brackets", FP.FP); //code style
        putDef(ret, "#jR;47;Diese Methode kann so nicht funktionieren, weil Betriebssysteme gleichzeitig case-sensitive und -unsensitive Dateisysteme mounten können", FP.FP);
        putDef(ret, "#jR;48;if the two paths are not equal ignoring case, you will always end with the error in the else clause", FP.FP); //code is OK
        putDef(ret, "#jR;48;path1", FP.FP); //no, the attribute name is path
        putDef(ret, "#jR;48;should be !path.equals(path2)", FP.FP); //no, algorithm is correct
        putDef(ret, "#jR;48;equalsIgnoreCase nutzt sehr wahrscheinlich nicht die gleiche Logik wie das zugrunde liegende Dateisystem; hier muss eine OS-API genutzt oder die Logik geändert werden (bspw.: inodes von _getFile(path) und _getFile(path2) vergleichen; VFS sollte das als isSameFile(path, path2) kapseln)", FP.FP);
        putDef(ret, "#jR;48;equalsIgnoreCase nutzt sehr wahrscheinlich nicht die gleiche Logik wie das zugrunde liegende Dateisystem; hier muss eine OS-API genutzt oder die Logik geändert werden", FP.FP);
        putDef(ret, "#jR;48;equalsIgnoreCase nutzt sehr wahrscheinlich nicht die gleiche Logik wie das zugrunde liegende Dateisystem; hier muss eine OS-API genutzt oder die Logik geändert werden (bspw.: _getFile(path) und _getFile(path2) vergleichen)", FP.FP);
        putDef(ret, "#jR;48;equalsIgnoreCase nutzt sehr wahrscheinlich nicht die gleiche Logik wie das zugrunde liegende Dateisystem; hier muss eine OS-API genutzt oder die Logik geändert werden (bspw.: inodes vergleichen, ...)", FP.FP);
        putDef(ret, "#jR;48;equalsIgnoreCase nutzt sehr wahrscheinlich nicht die gleiche Logik wie das zugrunde liegende Dateisystem; hier muss eine OS-API genutzt oder die Logik geändert werden (bspw.: inodes von _getFile(path) und _getFile(path2) vergleichen)", FP.FP);
        putDef(ret, "#jR;48;path2 were removed from super class", FP.FP);
        putDef(ret, "#jR;48;path is not declared", FP.FP);
        putDef(ret, "#jR;48;equals trifft hier auch zu. Der Fall gleicher Dateinamen müsste abgefangen werden.", FP.FP); //code is OK
        putDef(ret, "#jR;49;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;50;If it's not implemented it is not implemented. Check negative directly.", FP.FP); //code style
        putDef(ret, "#jR;50;valid block. but no executable statement.", FP.FP); //code style
        putDef(ret, "#jR;50;This part is strange, but it was that way before", FP.FP);
        putDef(ret, "#jR;50;code is incomplete here and will fail in line 62 as access to path2 is impossible", FP.FP);
        putDef(ret, "#jR;50;is the operation vfs._getFile(..)  case insensitive?", FP.FP);
        putDef(ret, "#jR;50;Is this a TODO? Will this part even work given that we use canonical paths and stuff?", FP.FP);
        putDef(ret, "#jR;50;not implemented TODO", FP.FP);
        putDef(ret, "#jR;50;avoid empty block", FP.FP);
        putDef(ret, "#jR;50;Hier fehlt eine Implementierung oder ein TODO", FP.FP); //wrong, is meant as fall-through
        putDef(ret, "#jR;50;die Implementierung fehlt", FP.FP); //wrong, is meant as fall-through
        putDef(ret, "#jR;50;empty block", FP.FP);
        putDef(ret, "#jR;50;nothing inside?", FP.FP);
        putDef(ret, "#jR;50;please implement", FP.FP); //wrong, is meant as fall-through
        putDef(ret, "#jR;53;else can append end of line of previous block to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;54;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;55;are you sure you didn't switch statements here?", FP.FP); //code is OK
        putDef(ret, "#jR;55;path1", FP.FP); //no, the attribute name is path
        putDef(ret, "#jR;55;path is not declared", FP.FP);
        putDef(ret, "#jR;55;If the file system is case sensitive, I think the user will expect to be able to change file names case sensitive.", FP.FP); //code is OK
        putDef(ret, "#jR;55;this error should be in true path.", FP.FP); //code is OK
        putDef(ret, "#jR;55;And what happens on a case sensitive system?", FP.FP); //code is OK
        putDef(ret, "#jR;58;finally block omitted", FP.FP); //I don't think so, perhaps misunderstanding of return semantics
        putDef(ret, "#jR;58;return statement must have variable to return", FP.FP);
        putDef(ret, "#jR;59;add case for case sensitive vfs", FP.FP);
        putDef(ret, "#jR;62;path1", FP.FP); //no, the attribute name is path
        putDef(ret, "#jR;62;Curly brackets", FP.FP); //code style
        putDef(ret, "#jR;63;path1", FP.FP); //no, the attribute name is path
        putDef(ret, "#jR;63;kein Logging hier?", FP.FP);
        putDef(ret, "#jR;64;path2 were removed from super class", FP.FP);
        putDef(ret, "#jR;67;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;68;Code Duplikat mit finally Block, vereinheitlichen.", FP.FP); //design issue
        putDef(ret, "#jR;68;kann hier raus, wenn unten im finally-Block", FP.FP);
        putDef(ret, "#jR;69;Nicer log messages? Let's discuss.", FP.FP);
        putDef(ret, "#jR;69;Same duplicated error handling", FP.FP); //design issue
        putDef(ret, "#jR;71;path1", FP.FP); //no, the attribute name is path
        putDef(ret, "#jR;71;path is not declared", FP.FP);
        putDef(ret, "#jR;71;directory?", FP.FP);
        putDef(ret, "#jR;71;directory->rename?", FP.FP);
        putDef(ret, "#jR;71;rename-error", FP.FP);
        putDef(ret, "#jR;71;Muss ioerror.rename-error statt ioerror.directory-error sein", FP.FP);
        putDef(ret, "#jR;71;Muss wahrscheinlich ioerror oder ioerror.rename-error statt ioerror.directory-error sein, aber aber vorher auch schon falsch durch Copy-Paste aus listDirectories() (?)", FP.FP);
        putDef(ret, "#jR;71;error string inconsistent", FP.FP);
        putDef(ret, "#jR;71;Muss wahrscheinlich ioerror.rename-error statt ioerror.directory-error sein, aber aber vorher auch schon falsch durch Copy-Paste aus listDirectories() (?)", FP.FP);
        putDef(ret, "#jR;71;Muss wahrscheinlich ioerror.rename-error statt ioerror.directory-error sein, aber aber vorher auch schon falsch durch Copy-Paste aus listDirectories()", FP.FP);
        putDef(ret, "#jR;71;Muss wahrscheinlich ioerror.rename-error statt ioerror.directory-error sein, aber aber vorher auch schon falsch(?)", FP.FP);
        putDef(ret, "#jR;71;In a case of an IOException, how do you want to distinguish the IOException in catch and finally clause?", FP.FP); //not a correctness defect
        putDef(ret, "#jR;73;InterruptedException-Behandlung fehlt", FP.FP); //would be compile error?
        putDef(ret, "#jR;74;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;76;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;81;Code Duplikat mit Catch-Block des äußeren try, vereinheitlichen.", FP.FP); //design issue
        putDef(ret, "#jR;81;code duplication", FP.FP); //design issue
        putDef(ret, "#jR;81;muss in finally", FP.FP);
        putDef(ret, "#jR;81;Duplicate error handling with line 66 -> Extract", FP.FP); //design issue
        putDef(ret, "#jR;82;same concern about undistinguishable log record with line 69", FP.FP); //ok, but not really a correctness issue
        putDef(ret, "#jR;83;No need for a variable here.", FP.FP); //code style
        putDef(ret, "#jR;84;directory?", FP.FP);
        putDef(ret, "#jR;84;directory->rename?", FP.FP);
        putDef(ret, "#jR;84;Muss ioerror.rename-error statt ioerror.directory-error sein", FP.FP);
        putDef(ret, "#jR;84;path is not declared", FP.FP);
        putDef(ret, "#jR;84;In a case of an IOException, how do you want to distinguish the IOException in catch and finally clause?", FP.FP); //not a correctness defect
        putDef(ret, "#jR;84;You use the same piece of code also above: you could move it inside a function and reuse it", FP.FP); //design issue
        putDef(ret, "#jR;84;same concern about undistinguishable calls from here and line 71", FP.FP); //ok, but no correctness defect
        putDef(ret, "#jR;84;error string inconsistent", FP.FP);
        putDef(ret, "#jR;84;Muss wahrscheinlich ioerror oder ioerror.rename-error statt ioerror.directory-error sein, aber aber vorher auch schon falsch durch Copy-Paste aus listDirectories() (?)", FP.FP);
        putDef(ret, "#jR;84;Muss wahrscheinlich ioerror.rename-error statt ioerror.directory-error sein, aber aber vorher auch schon falsch durch Copy-Paste aus listDirectories() (?)", FP.FP);
        putDef(ret, "#jR;84;Muss wahrscheinlich ioerror.rename-error statt ioerror.directory-error sein, aber aber vorher auch schon falsch durch Copy-Paste aus listDirectories()", FP.FP);
        putDef(ret, "#jR;84;Muss wahrscheinlich ioerror.rename-error statt ioerror.directory-error sein, aber aber vorher auch schon falsch(?)", FP.FP);
        putDef(ret, "#jR;86;Assuming that setCancellable is equivalent to the original setAbortable a setCancellable might be missing in the no error case (after the finally) because it is present in listDirectory but not in the other replaced methods", FP.FP);
        putDef(ret, "#jR;87;same concern about setCancellable(false) whihc should be better noved into the finally block", FP.FP);
        putDef(ret, "#jR;88;", FP.FP);
        putDef(ret, "#jR;90;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;91;Let's discuss all of that string concatenation. Better to put this outside in a database, file or whatever.", FP.FP); //code style?
        putDef(ret, "#jR;92;path1", FP.FP); //no, the attribute name is path
        putDef(ret, "#jR;92;fromPath=path toPath=path2 ?", FP.FP); //requirements issue
        putDef(ret, "#jR;93;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#jR;93;path1 were removed super class", FP.FP); //?
        putDef(ret, "#jR;93;path1 and path2 were removed super class", FP.FP);
        putDef(ret, "#jR;93;Not a bug but when there is path2 you expect to see path1 somewhere not just path", FP.FP); //naming
        putDef(ret, "#jR;96;path2 is also defined in run(), is this a problem?", FP.FP); //no, it is not
        putDef(ret, "#jR;96;Felder bitte ganz oben in der Klasse deklarieren.", FP.FP); //code style (or did not understand re-ordering?)
        putDef(ret, "#jR;96;variable should be at beginning of class", FP.FP); //code style
        putDef(ret, "#jR;96;move to top of class for better readability", FP.FP); //code style
        putDef(ret, "#jR;96;Shouldn't that be in a different place?", FP.FP); //code style
        putDef(ret, "#jR;96;name is not expressive, better rename it to \"newPathName\" or something", FP.FP); //naming
        putDef(ret, "#kR;45;missing Javadoc for params \"session\", \"vfs\" and \"awtRunnable\"", FP.FP); //documentation
        putDef(ret, "#kR;46;Bad parameter documentation. Does not explain the purpose of the parameter", FP.FP); //documentation
        putDef(ret, "#kR;46;The path name to operate on", FP.FP); //documentation
        putDef(ret, "#kR;46;The only path name to operate on", FP.FP); //documentation
        putDef(ret, "#kR;46;The \"first\" in the comment is misleading. Shouldn't it be \"the path name to operate on\"?", FP.FP); //documentation
        putDef(ret, "#kR;46;\"first\" should be removed", FP.FP); //documentation
        putDef(ret, "#kR;46;javadoc incomplete", FP.FP); //documentation
        putDef(ret, "#kR;47;Bad parameter documentation, format of array is unclear", FP.FP); //documentation
        putDef(ret, "#kR;48;Element 1/2 oder 0/1? #_run setzt Element 0 und 1, und so sollte es auch dokumentiert werden.", FP.FP); //documentation
        putDef(ret, "#kR;49;Parameter description for awtRunnable missing", FP.FP); //documentation
        putDef(ret, "#kR;50;Missing Javadoc for awtRunnable", FP.FP); //documentation
        putDef(ret, "#kR;52;loadInfo is not assigned to the corresponding field", P1.LOADINFO_NOT_SET);
        putDef(ret, "#kR;52;loadInfo is obvoisuly not used any more. Do we need it as parameter?", P1.LOADINFO_NOT_SET);
        putDef(ret, "#kR;52;loadInfo is not used", P1.LOADINFO_NOT_SET);
        putDef(ret, "#kR;52;\"loadInfo\" passed as parameter but ignored", P1.LOADINFO_NOT_SET);
        putDef(ret, "#kR;52;The giben Object array loadInfor isnt used anymore. It should be deleted", P1.LOADINFO_NOT_SET);
        putDef(ret, "#kR;53;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#kR;54;why are the additional parameters removed from the base ctor, but not from ListDirectoryBrowserTask()?", FP.FP); //because the field is moved to the subclass
        putDef(ret, "#kR;54;loadInfo is not assigned", P1.LOADINFO_NOT_SET);
        putDef(ret, "#kR;54;Missing null-parameter and loadInfo parameter", P1.LOADINFO_NOT_SET);
        putDef(ret, "#kR;54;Suspect: How can the number of parameters of the super()-call change?", FP.FP); //why not?
        putDef(ret, "#kR;54;the loadInfo field is not initialized.", P1.LOADINFO_NOT_SET);
        putDef(ret, "#kR;54;check init of loadInfo", P1.LOADINFO_NOT_SET);
        putDef(ret, "#kR;54;Die Zuweisung von loadInfo fehlt, so wird #_run in einer NPE enden", P1.LOADINFO_NOT_SET);
        putDef(ret, "#kR;54;loadInfo is not used; false super call?", P1.LOADINFO_NOT_SET);
        putDef(ret, "#kR;55;toString method has to be rewritten", FP.FP); //and why?
        putDef(ret, "#lR;42;loadInfo sollte im Konstruktor auch gesetzt werden mit dem übergebenen Parameter.", P1.LOADINFO_NOT_SET);
        putDef(ret, "#lR;42;i wonder how this class compiles as load info is final but not assigned in the constructor", P1.LOADINFO_NOT_SET);
        putDef(ret, "#lR;42;comment missing? what is this Object-Array good for?", FP.FP); //documentation
        putDef(ret, "#lR;42;Wird dieses Feld im Konstruktor definiert?", P1.LOADINFO_NOT_SET);
        putDef(ret, "#lR;42;no setter/no use?", P1.LOADINFO_NOT_SET);
        putDef(ret, "#lR;42;I think this is not actually used now", P1.LOADINFO_NOT_SET); //with some goodwill
        putDef(ret, "#lR;42;loadInfo is declared in this class, but never set. was propagated to base ctor before, but neither is nor is it set in ctor now.", P1.LOADINFO_NOT_SET);
        putDef(ret, "#lR;42;forgot to set loadInfo in constructor?", P1.LOADINFO_NOT_SET);
        putDef(ret, "#mR;61;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#mR;65;is this correct? Which version of canonPath will be contained in the loadInfo array after an error occurred?", FP.FP); //we assume that this works as designed
        putDef(ret, "#mR;65;Can be declared more locally.", FP.FP); //code style
        putDef(ret, "#mR;65;Variale should be declared in next try block, because it is not used outside of it", FP.FP); //not true
        putDef(ret, "#mR;65;Unnötige/verwirrende Zuweisung: Der Wert wird direkt darunter mit dem Ergebnis von _canonPath überschrieben", FP.FP);
        putDef(ret, "#mR;69;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#mR;72;wurde eben mit path initialisiert", FP.FP); //design issue/not a problem
        putDef(ret, "#mR;72;after path, a space", FP.FP); //code style
        putDef(ret, "#mR;72;Whitespace after comma", FP.FP); //code style
        putDef(ret, "#mR;72;Why is the path field not overwritten with the canonical path like in rename task for example?", FP.FP); //right, but no correctness defect
        putDef(ret, "#mR;72;Only in this class \"path\" class field not updated", FP.FP); //right, but no correctness defect
        putDef(ret, "#mR;73;Diese Zuweisung wird nie benutzt oder?", FP.FP); //not true
        putDef(ret, "#mR;75;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#mR;76;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#mR;77;consider moving into finally block to ensure it is called once", FP.FP);
        putDef(ret, "#mR;79;pp nicht sprechender Variablenname", FP.FP); //coding style
        putDef(ret, "#mR;80;How do you want to distinguish the error in the catch and the finally clause?", FP.FP); //not a correctness defect
        putDef(ret, "#mR;80;als Static String machen und die anderen Strings auch", FP.FP); //code style
        putDef(ret, "#mR;80;again space after path,", FP.FP); //code style
        putDef(ret, "#mR;80;Hier wird im Gegensatz zu den anderen BrowserTasks der path, nicht der kanonische path geloggt - vielleicht projektweit vereinheitlichen, um Überraschungen zu vermeiden", FP.FP); //we don't count this as defect
        putDef(ret, "#mR;81;Whitespace after comma", FP.FP); //code style
        putDef(ret, "#mR;81;InterruptedException-Behandlung?", FP.FP); //?
        putDef(ret, "#mR;83;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#mR;85;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#mR;89;could not start with the open braces to avoid one new line", FP.FP); //code style
        putDef(ret, "#mR;90;code duplication. do the catch elsewhere", FP.FP); //design issue
        putDef(ret, "#mR;90;same error handling", FP.FP); //design issue
        putDef(ret, "#mR;90;again, copy-pasted, failue from finally block should not hide the main failure", FP.FP); //design issue
        putDef(ret, "#mR;91;make distinguishable from log record created at line 78", FP.FP); //no correctness defect
        putDef(ret, "#mR;92;pp nicht sprechender Variablenname", FP.FP);
        putDef(ret, "#mR;93;space", FP.FP); //code style
        putDef(ret, "#mR;93;How do you want to distinguish the error in the catch and the finally clause?", FP.FP); //not a correctness defect
        putDef(ret, "#mR;93;Whitespace after comma", FP.FP); //code style
        putDef(ret, "#mR;97;How do you distinguish in the cancellable flag whether the task was successful executed or an exception occurred?", FP.FP); //design issue
        putDef(ret, "#mR;97;consider moving into finally block to ensure it is called once", FP.FP); //no need to call it because method ends directly afterward
        putDef(ret, "#nR;107;Zeichenkonstante evtl. nicht richtig benannt, Sollte diese nicht path heißen?", P1.TYPO_PATH);
        putDef(ret, "#nR;107;\",path=\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;path not pat", P1.TYPO_PATH);
        putDef(ret, "#nR;107;Typo: \"path\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;Typo: should be \"path\" not \"pat\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;You have a type in \"pat=\". It should be \"path=\".", P1.TYPO_PATH);
        putDef(ret, "#nR;107;name \"pat\" to \"path\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo in String literal", P1.TYPO_PATH);
        putDef(ret, "#nR;107;pat should be path.", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo", P1.TYPO_PATH);
        putDef(ret, "#nR;107;should be path not pat", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo error in label \"pat\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;pat -> path", P1.TYPO_PATH);
        putDef(ret, "#nR;107;path= (missing h)", P1.TYPO_PATH);
        putDef(ret, "#nR;107;Typo: \",pat=\" should be \", path=\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo error \"pat\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;,path h missing", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo? Consider using string constants to be sure keys are consistent", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo path.", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo: \"pat\" instead of \"path\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo?", P1.TYPO_PATH);
        putDef(ret, "#nR;107;missing h in path", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo path. why is path instead of path1 used?", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo: pat->path", P1.TYPO_PATH);
        putDef(ret, "#nR;107;Message muss path enthalten nicht pat", P1.TYPO_PATH);
        putDef(ret, "#nR;107;type path*", P1.TYPO_PATH);
        putDef(ret, "#nR;107;Typo", P1.TYPO_PATH);
        putDef(ret, "#nR;107;path is not declared any where", FP.FP); //it is
        putDef(ret, "#nR;107;typo, missing space", P1.TYPO_PATH);
        putDef(ret, "#nR;107;\"path\"?", P1.TYPO_PATH);
        putDef(ret, "#nR;107;mistyping \"pat=\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;Typo \",path=\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;Spelling error \"pat\", should be \"path\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;path and not pat", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo pat", P1.TYPO_PATH);
        putDef(ret, "#nR;107;Typo pat=", P1.TYPO_PATH);
        putDef(ret, "#nR;107;Typo: path", P1.TYPO_PATH);
        putDef(ret, "#nR;107;path", P1.TYPO_PATH);
        putDef(ret, "#nR;107;type pat[h]", P1.TYPO_PATH);
        putDef(ret, "#nR;107;Typo: pat should be path", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo: pat= should be path=", P1.TYPO_PATH);
        putDef(ret, "#nR;107;type, should be \"path\" instead of \"pat\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;typo in pat=", P1.TYPO_PATH);
        putDef(ret, "#nR;107;", P1.TYPO_PATH); //with lots of goodwill
        putDef(ret, "#nR;107;typo: pat - >path", P1.TYPO_PATH);
        putDef(ret, "#nR;107;pasth", P1.TYPO_PATH);
        putDef(ret, "#nR;107;check param name \"pat\"", P1.TYPO_PATH);
        putDef(ret, "#nR;107;Typo: pat", P1.TYPO_PATH);
        putDef(ret, "#nR;107;should be called path", P1.TYPO_PATH);
        putDef(ret, "#nR;107;misspelling \"pat\"", P1.TYPO_PATH);
        putDef(ret, "#oL;113;listDirectory was removed but it does not seem to have replacement.", FP.FP); //was already replaced in previous ticket
        putDef(ret, "#hR;44;muss ins finally", FP.FP);
        putDef(ret, "#hR;57;ab in finally!", FP.FP);
        putDef(ret, "#mR;65;Warum wird canonPath hier belegt, dieser Wert nie genutzt und dann später überschrieben?", FP.FP);
        return ret;
    }
}