package de.unihannover.orderingExperimentAnalysis;

import java.io.IOException;
import java.util.Collection;

import de.unihannover.orderingExperimentAnalysis.SessionData.Defect;

public class PrintDefectTiming {

    public static void main(String[] args) throws Exception {
        try {
            print(AnalyzeExperimentLogs.getFiles());
        } catch (final Exception e) {
            System.out.println();
            e.printStackTrace(System.out);
        }
    }

    private static void print(String... filenames) throws IOException {
        final LogRecords log = LogRecords.load(filenames);
        final Collection<SessionData> sessions = log.getSplitByParticipant();

        sessions.removeIf(SessionData::isJeditIncomplete);
        sessions.removeIf(SessionData::isDuplicateParticipation);
        sessions.removeIf((SessionData s) -> !(s.isRealEffort(1) || s.isRealEffort(2)));

        System.out.println("id;change;treatment;reviewNumber;defect;foundAfter;totalReviewDuration");
        for (final SessionData s : sessions) {
            for (int number = 0; number < 3; number++) {
                final long totalDuration = s.getNetTimeForReview(number, true);
                for (final Defect defect : s.getAllDefectsForReview(number)) {
                    final long timeForDefect = s.getNetTimeUntilFound(number, defect);
                    if (timeForDefect >= 0) {
                        System.out.println(
                                    s.getId() + ";" +
                                    s.getChange(number) + ";" +
                                    s.getTreatment(number) + ";" +
                                    number + ";" +
                                    defect.getClass().getSimpleName() + "_" + defect + ";" +
                                    timeForDefect + ";" +
                                    totalDuration);
                    }
                }
            }
        }
    }
}
