package de.unihannover.orderingExperimentAnalysis;

import static de.unihannover.orderingExperimentAnalysis.SessionData.Defect.putDef;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import de.unihannover.orderingExperimentAnalysis.SessionData.Defect;
import de.unihannover.orderingExperimentAnalysis.SessionData.FP;

public enum WU implements Defect {
    /**
     * From and to swapped in method call.
     */
    FROM_TO_SWAP,
    /**
     * Inconsistent check of from index, upper bound is checked as zero-based but lower-bound is checked as one-based.
     */
    OFF_BY_ONE1,
    /**
     * Inconsistent check of to index, upper bound is checked as zero-based but lower-bound is checked as one-based.
     */
    OFF_BY_ONE2;

    @Override
    public String getShortString() {
        return this.getClass().getSimpleName() + "_" + this.ordinal();
    }

    @Override
    public boolean isDelocalized() {
        return this == FROM_TO_SWAP;
    }


    static Map<String, List<Defect>> getTrueDefects() {
        final Map<String, List<Defect>> ret = new LinkedHashMap<>();
        putDef(ret, "#0R;59;{ open braces may come at end of line for reduce one line", FP.FP); //code style
        putDef(ret, "#0R;60;does table inherit from tableModel?!", FP.FP);
        putDef(ret, "#0R;67;bad style, ste renderer before call to setDefaultRenderer()", FP.FP); //style
        putDef(ret, "#0R;67;don't give a parameter a value within callin", FP.FP); //code style
        putDef(ret, "#0R;69;is it nullsafe?", FP.FP);
        putDef(ret, "#0R;70;default is true", FP.FP);
        putDef(ret, "#0R;70;Sorry do not understand why reordering should be allowed now.", FP.FP);
        putDef(ret, "#0R;70;introduce external property to change this value without redeployment", FP.FP);
        putDef(ret, "#0R;70;No enough information about the use case which should be implemented", FP.FP);
        putDef(ret, "#0R;70;'o' shouold be in capital on setReorderingAllowed method", FP.FP); //code style
        putDef(ret, "#0R;70;The general problem is that there is not commit message attached to this sample. There for is hard to say anything about the change as there is not enoguht context provided. Usually the change is self is not so importat as the way its done.", FP.FP);
        putDef(ret, "#0R;71;useless", FP.FP);
        putDef(ret, "#0R;74;does headerRenderer inherit from tableRenderer?", FP.FP);
        putDef(ret, "#1R;275;Write a comment to explain what \"from\" and \"to\" means. Why are they not \"uint\"? Write in the comment that they should start from one!", FP.FP); //documentation
        putDef(ret, "#1R;275;Unclear method description. Nothing is returned.", FP.FP); //documentation
        putDef(ret, "#1R;275;What about \"from > to\"", FP.FP); //is OK
        putDef(ret, "#1R;275;Why this is private? Also the mane should be defining that is done on the object it should be moveColumn and return true if success or false if not.", FP.FP);
        putDef(ret, "#1R;275;nullsafety?", FP.FP);
        putDef(ret, "#1R;275;What if from > to?", FP.FP);
        putDef(ret, "#1R;275;(0;n] is bad interval for index. it should be [0,n). all fixes below for current behavior", FP.FP);
        putDef(ret, "#1R;275;Consider restricting visibility to \"package-private\"", FP.FP);
        putDef(ret, "#1R;275;unclear if it is 0 or 1 based indexing. If statements suggest 1 based. ea variable suggest 0 based.", WU.OFF_BY_ONE1, WU.OFF_BY_ONE2);
        putDef(ret, "#1R;275;In general, it would make more sense to have \"from\"/\"to\" go from 0 to max-1 instead of 1 to max", FP.FP); //design issue
        putDef(ret, "#1R;275;make public?", FP.FP);
        putDef(ret, "#1R;276;is checking of from and to index ranges really necessary?", FP.FP); //design issue
        putDef(ret, "#1R;276;Externilized validtion to separete method", FP.FP); //design issue
        putDef(ret, "#1R;276;is checking of from and to index really necessary?", FP.FP);
        putDef(ret, "#1R;276;Variables \"to\" and \"from\" can easily be confused. I's suggest a value object that encapsulated the desired movement. Refactor this method to use the new value object instead auf primitve parameters. Also: Validation should then be handled inside the value object.", FP.FP); //design issue
        putDef(ret, "#1R;276;All of the if expressions can be chained together by ||.", FP.FP); //code style
        putDef(ret, "#1R;276;add more parantheses for a more readable structure, even for single line blocks", FP.FP); //code style
        putDef(ret, "#1R;276;Do this in one line. Is easier to understand. Write a comment.", FP.FP); //code style
        putDef(ret, "#1R;277;Ist es evtl. möglich zu unterbinden, dass die Methode in dem Fall gerufen wird?", FP.FP);
        putDef(ret, "#1R;277;Make here a clear exit path with one exit.", FP.FP); //code style
        putDef(ret, "#1R;277;variable missing after return", FP.FP); //code style
        putDef(ret, "#1R;278;assuming one-based indices, the second check must be \">\", not \">=\" (if zero-based indices are to be used, the first check must be \"< 0\", not \"< 1\", and the element access below must be adapted accordingly)", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;condition does not match indexes of ExtendedAttribute which seem to be starting from 0 but are treated like 1", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;muss: from > getColum...", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;check valid indices", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;(0;n] is bad interval for index. it should be [0,n)", FP.FP);
        putDef(ret, "#1R;278;This if-statement excludes either the first or last column, depending on whether from is 0 or 1-based.", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;typically 0-boudned", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;from could be equal to 0, why that case is not considered?", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;Indices should be used consistently. Here the from-index is 1-based, but in extAttrs it seems to be 0-based.", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;(from < 1): Auf diese Weise kann die 0te-Spalte nie verschoben werden.  (from >= getColumnCount()): falls das autritt, sollte der Vorfall nicht verschluckt werden, denn das sollte nicht vorkommen.", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;why is index 0 also not valid?", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;column Moved is called with Indexes. They are zero Based ...", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;use else if", FP.FP); //code style
        putDef(ret, "#1R;278;If the from and to start at 1 at go to max, we will need (from > getColumnCount). If from goes from 0 to max-1 we will need (from < 0) as a condition", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;from ist 0-basiert, Abfrage muss also <0 statt <1 sein", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;\"from\" is probably zero based, so it should be \"< 0\" and not \"< 1\".", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;what about from column position 0?", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;assuming one-based indices, the second check must be \">\", not \">=\"", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;Sind die Indizes tatsächlich 1 basiert oder doch 0 basiert?", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;assuming that from/to really are 1-based index then the from >= should be from >", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;This if-statement excludes the last column, it should test for > getColumnCount().", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;Grenzen nicht konsistent", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;(from < 1): Auf diese Weise kann die 0te-Spalte nie verschoben werden, da der aufrufende Code mit dem Index arbeitet. Ist das Absicht? (from >= getColumnCount()): falls das autritt, sollte der Vorfall nicht verschluckt werden, denn das wäre ein Fehlerfall.", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;Indices should be used consistently. Here the from-index is 1-based, but in extAttrs it seems to be 0-based. Does getColumnCount() also use extAttrs to detemine the column count or may they be out of sync?", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;column numeration starts from 1 or 0?", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;is from zero-based?", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;(from < 1): Auf diese Weise kann die 0te-Spalte nie verschoben werden, da mit dem Index gearbeitet wird. Ist das Absicht? (from >= getColumnCount()): falls das autritt, sollte der Vorfall nicht verschluckt werden, denn das sollte nicht vorkommen.", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;(from < 1): Auf diese Weise kann die 0te-Spalte nie verschoben werden. Ist das Absicht? (from >= getColumnCount()): falls das autritt, sollte der Vorfall nicht verschluckt werden, denn das sollte nicht vorkommen.", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;from ist 0-basiert, Abfrage muss also <0 statt <1 sein (wenn die Indizes 1-basiert sind, ist wiederum `from >= getColumnCount` falsch, es müsste dann > sein)", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;Must be 0 not 1", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;If columns get counted beginnging from 0 this is a bug since the first column with the number 0 can then not be moved", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;As column numbers seem to be 1-based, this has to be `from > getCloumtCount()`", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;Borders seem to be not consistent; Assuming we have 5 columns (0 to 4), first criteria should be change to \"<0\" from \"<1\"", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;getColumnCount() does not exist?", FP.FP);
        putDef(ret, "#1R;278;it isn't clear if indexing is starting from 0 or 1. If it's 0 then \"<1\" must be replaced with \"<0\", otherwise \">= count\" must be replaced with \">count\"", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;possible index 0 dismissed, should have been < 0 probably", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;statt \"from >=..\" muss  \"to>=..\" sein", WU.OFF_BY_ONE1); //line is correct but comment is wrong, we count it anyway
        putDef(ret, "#1R;278;check: is the index zero-based?", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;from and to are indices: from should not be smaller than 0, but 0 is ok", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;Can't the first column be moved?", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;If the from/to to starts at 1 at go to max, we will need (from > getColumnCount). If from/to goes from 0 to max-1 we will need (from < 0) as a condition", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;(from > getColumnCount) seems to be more legit", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;>= seems to be wrong and should be replaced by a simpe >", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;The lower bound should be < 0 instead of < 1 because 0 is still a valid column index", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;we need > here not >=", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;Col Pos 0 not possible?", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;from==0 is a correct Index. Correct: from <0", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;Assuming index based on 1, getColumCount should be allowed (> instead of >=)", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;278;typically 0-bounded; should be < 0", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;279;Sollte ja sicher nicht vorkommen, -> IllegalArgExc?", FP.FP); //design issue
        putDef(ret, "#1R;279;throw exception", FP.FP);
        putDef(ret, "#1R;279;Maybe here we should throw exception or log error message instead of ignoring incorrect request", FP.FP); //design issue
        putDef(ret, "#1R;279;variable missing after return", FP.FP); //code style
        putDef(ret, "#1R;279;Consider throwing IllegalArgumentException instead of just returning silently", FP.FP);
        putDef(ret, "#1R;280;s.o. Exc?", FP.FP); //design issue
        putDef(ret, "#1R;280;assuming one-based indices, the second check must be \">\", not \">=\" (if zero-based indices are to be used, the first check must be \"< 0\", not \"< 1\", and the element access below must be adapted accordingly)", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;(0;n] is bad interval for index. it should be [0,n)", FP.FP);
        putDef(ret, "#1R;280;see line 278", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;Can't any column be moved to the place of the first one?", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;muss: to> getColum...", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;is to zero-based?", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;Die erste Spalte lässt sich nicht setzen. Ist das Absicht?", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;Must be 0 not 1", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;Indices should be used consistently. Here the to-index is 1-based, but in extAttrs it seems to be 0-based.", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;Col Pos 0 not possible?", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;This if-statement excludes either the first or last column, depending on whether to is 0 or 1-based.", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;assuming one-based indices, the second check must be \">\", not \">=\"", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;check valid indices", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;to ist 0-basiert, Abfrage muss also <0 statt <1 sein", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;what about when moving a column back to column position 0... i.e the first?", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;to ist 0-basiert, Abfrage muss also <0 statt <1 sein (wenn die Indizes 1-basiert sind, ist wiederum `to >= getColumnCount` falsch, es müsste dann > sein)", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;Duplicate code with line 278", FP.FP); //code style
        putDef(ret, "#1R;280;to could be equal to 0, why that case is not considered?", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;>= should be replaced by a simple >", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;typically 0-bounded. Should be <0", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;why is index 0 also not valid?", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;analog to line 278", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;Same as in line 278 if you start counting columns from 0. With this code you can not move column to the first one", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;from and to are indices: to should not be smaller than 0, but 0 is ok", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;what about when moving a column back to column position 0... i.e the first position in the table?", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;\"to\" is probably zero based, so it should be \"< 0\" and not \"< 1\".", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;Sind die Indizes tatsächlich 1 basiert oder doch 0 basiert?", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;it isn't clear if indexing is starting from 0 or 1. If it's 0 then \"<1\" must be replaced with \"<0\", otherwise \">= count\" must be replaced with \">count\"", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;(from > getColumnCount) seems to be more legit", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;Grenzen nicht konsistent", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;The lower bound should be < 0 instead of < 1 because 0 is still a valid column index", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;Die erste Spalte lässt sich nicht setzen, da mit dem Index gearbeitet wird. Ist das Absicht?", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;Indices should be used consistently. Here the to-index is 1-based, but in extAttrs it seems to be 0-based. Does getColumnCount() also use extAttrs to detemine the column count or may they be out of sync?", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;possible index 0 dismissed, should have been < 0 probably", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;löschen", WU.OFF_BY_ONE2); //line is correct but comment is wrong, we count it anyway
        putDef(ret, "#1R;280;Same here, if 1-based, then `to > getColumnCount()`", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;This if-statement excludes the last column, it should test for > getColumnCount().", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;we need > here not >=", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;assuming that from/to really are 1-based index then the to >= should be to >", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;index zero-based? see above", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;to <0 (see last remark)", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;280;see above", WU.OFF_BY_ONE2);
        putDef(ret, "#1R;281;variable missing after return", FP.FP); //code style
        putDef(ret, "#1R;281;Return probably AFTER ea-manipulation", FP.FP); //no, is correct as a premature return
        putDef(ret, "#1R;281;Consider throwing IllegalArgumentException instead of just returning silently", FP.FP);
        putDef(ret, "#1R;281;from instead of from-1", FP.FP);
        putDef(ret, "#1R;281;löschen", FP.FP);
        putDef(ret, "#1R;281;Curly braces around the if statements", FP.FP); //code style
        putDef(ret, "#1R;281;throw exception", FP.FP);
        putDef(ret, "#1R;281;merge to one if", FP.FP);
        putDef(ret, "#1R;282;Evtl. problematisch (s. unten)", WU.OFF_BY_ONE1);
        putDef(ret, "#1R;282;It's better to start the ranges from 0..x instead of 1..x. (see other remark above). Then you can get rid of the \" - 1\".", FP.FP); //design issue
        putDef(ret, "#1R;282;see line 278", FP.FP);
        putDef(ret, "#1R;282;use from", FP.FP);
        putDef(ret, "#1R;282;from and to are handled with lower bound 1 here. I think the TableModels do not do this.", FP.FP);
        putDef(ret, "#1R;282;This will remove a wrong attribute. Should be extAttrs.remove(from)", FP.FP);
        putDef(ret, "#1R;282;Je nachdem, ob nsch links oder rechts verschoben wird, ändert sich die Zielposition. Das wird hier nicht berücksichtigt", FP.FP);
        putDef(ret, "#1R;282;First add, then remove. Correct indexing: remove(from)", FP.FP);
        putDef(ret, "#1R;282;Wenn man davon ausgeht, dass mit dem Index gearbeitet wird, so wird auf diese Weise immer die Spalte links neben der adressierten verschoben.", FP.FP);
        putDef(ret, "#1R;282;\"extAttrs\" is not initialized (but can be a field)", FP.FP); //yes, is a field
        putDef(ret, "#1R;282;assuming extAttrs has a list like interface: the algortithm only works if from > to", FP.FP);
        putDef(ret, "#1R;282;If extAttrs isn't a default java collection or list, we should implement a move-Method in the class of extAttrs type to prevent side effects. If parallel changes in extAttrs are possible, check correct synchronization. Other threads should only be able to change extAttrs after moving the column.", FP.FP); //design issue
        putDef(ret, "#1R;282;from instead of from-1", FP.FP);
        putDef(ret, "#1R;282;why -1?", FP.FP);
        putDef(ret, "#1R;282;we shouldnt subtract 1", FP.FP);
        putDef(ret, "#1R;282;from and to are indices: from should not be subtracted by 1", FP.FP);
        putDef(ret, "#1R;282;handling for to<1 is missing plus handling for from <1 is missing just return in  both cases plus handling for on of the parameters beeing >= count ..", FP.FP);
        putDef(ret, "#1R;282;handling for to<1 is missing plus handling for from <1 is missing just return in  both cases", FP.FP);
        putDef(ret, "#1R;282;why is there an offset of 1?", FP.FP);
        putDef(ret, "#1R;282;Isn't it better to have column numbering from 0 to getColumnCount -1?", FP.FP); //design issue
        putDef(ret, "#1R;282;Return probably AFTER ea-manipulation", FP.FP);
        putDef(ret, "#1R;282;Why we remove \"from-1\", not just \"from\" Apparently, extAttrs is a list, so we could just use the original \"from\" and \"to\" indexes, not subtracting \"-1\"", FP.FP);
        putDef(ret, "#1R;282;if you start counting columns from 0. You get negative numbers", FP.FP);
        putDef(ret, "#1R;282;What is \"extAttrs\" in this context? Is there a variable missing?", FP.FP);
        putDef(ret, "#1R;282;Wenn extAttrs 0-basiert ist, muss `from` statt `from - 1` gelöscht werden", FP.FP);
        putDef(ret, "#1R;282;describe index operation -> method extraction?", FP.FP);
        putDef(ret, "#1R;282;Wenn extAttrs 0-basiert ist, muss `from` statt `from - 1` gelöscht werden", FP.FP);
        putDef(ret, "#1R;283;Wenn extAttrs 0-basiert ist, muss bei `to` statt `to - 1` eingefügt werden", FP.FP);
        putDef(ret, "#1R;282;extAttrs ist not defined", FP.FP);
        putDef(ret, "#1R;282;\"from\" is probably zero based, so \"- 1\" chooses the wrong column or results in an IndexOutOfBoundsException.", FP.FP);
        putDef(ret, "#1R;282;Sind die Indizes tatsächlich 1 basiert oder doch 0 basiert?", FP.FP);
        putDef(ret, "#1R;282;Warum wird hier `from-1` statt `from` übergeben?  Ist extAttrs nicht auch 0-basiert?", FP.FP);
        putDef(ret, "#1R;282;Are the table model indices 0-based? If so, this whole method needs to be adjusted", FP.FP);
        putDef(ret, "#1R;282;Check whether indexes might be off due to removal", FP.FP);
        putDef(ret, "#1R;282;if column count is zero-based, then \"from-1\" resp \"to-1\" access illegal column when from == 0 resp \"to == 0\"", FP.FP);
        putDef(ret, "#1R;282;Where does \"extAttrs\" come from? Global (shared) state?", FP.FP);
        putDef(ret, "#1R;282;What is \"ea\"? Better name", FP.FP); //naming
        putDef(ret, "#1R;282;extAttrs ist 0-basiert ist, Argument muss `from` statt `from - 1` sein", FP.FP);
        putDef(ret, "#1R;282;If from \"less\" than \"to\" then removing \"from\" cause to change index of element which was accessed by \"to\" index", FP.FP);
        putDef(ret, "#1R;283;see line 278", FP.FP);
        putDef(ret, "#1R;283;Warum wird hier `to-1` statt `to` übergeben?  Ist extAttrs nicht auch 0-basiert?", FP.FP);
        putDef(ret, "#1R;283;to instead of to-1", FP.FP);
        putDef(ret, "#1R;283;See last remark", FP.FP);
        putDef(ret, "#1R;283;do to removing an element in the previous line the add might use the wrong index depending on if the from < to or not", FP.FP);
        putDef(ret, "#1R;283;siehe oben", FP.FP);
        putDef(ret, "#1R;283;we shouldnt subtract 1", FP.FP);
        putDef(ret, "#1R;283;describe index operation -> method extraction?", FP.FP);
        putDef(ret, "#1R;283;extAttrs ist 0-basiert, Argument muss `to` statt `to - 1` sein", FP.FP);
        putDef(ret, "#1R;283;use to, because it is more understandable and index starts with 0", FP.FP);
        putDef(ret, "#1R;283;\"to\" is probably zero based, so \"- 1\" chooses the wrong colum or results in an IndexOutOfBounds.", FP.FP);
        putDef(ret, "#1R;282;Why handle with +1 and here handle with -1? Everything 0 oder 1 based, no translation in head.", FP.FP);
        putDef(ret, "#1R;283;If an ea is removed first and is moved further to the left, to-1 is wrong.", FP.FP);
        putDef(ret, "#1R;283;if \"from\" < \"to\" it will be added at another position than if \"from\" > \"to\"", FP.FP);
        putDef(ret, "#1R;283;Da mit dem Index gearbeitet wird, wird auch hier die Spalte auf die Position links neben der adressierten verschoben.", FP.FP);
        putDef(ret, "#1R;283;Why we add to \"to-1\", not just \"from\" Apparently, extAttrs is a list, so we could just use the original \"from\" and \"to\" indexes, not subtracting \"-1\"", FP.FP);
        putDef(ret, "#1R;283;extAttrs.add(from < to ? to - 2 : to -1, ea);", FP.FP);
        putDef(ret, "#1R;283;if \"from\" < \"to\" it will be added at another position than if \"from\" > \"to\", due to remove() before", FP.FP);
        putDef(ret, "#1R;283;why is there an offset of 1? Also: if from is less than to, it's another one-off problem.", FP.FP);
        putDef(ret, "#1R;283;when from < to the `to` index should be adjusted for existing deletion", FP.FP);
        putDef(ret, "#1R;283;You have to adjust the position of to here, because the length of extAttrs has changed", FP.FP);
        putDef(ret, "#1R;283;0 oder 1 handle (see line282)", FP.FP);
        putDef(ret, "#1R;283;\"add\" will put \"ea\" in wrong position if \"from < to\"", FP.FP);
        putDef(ret, "#1R;283;\"to\" index might have been affected by \"remove\" above", FP.FP);
        putDef(ret, "#1R;283;to<from || t>from", FP.FP); //don't know what that means
        putDef(ret, "#1R;283;check if moving from higher index to lower index and vice versa works", FP.FP);
        putDef(ret, "#2R;574;Try to find an abstract class to not implement not used methods.", FP.FP);
        putDef(ret, "#2R;575;{ open braces may come at end of line for reduce one line", FP.FP); //code style
        putDef(ret, "#2R;578;last 2 args should be swapped around", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;578;Maybe you should check whether reordering columns is set to true (If not done before).", FP.FP);
        putDef(ret, "#2R;579;what does getModel return? You should verify the type", FP.FP);
        putDef(ret, "#2R;579;Parameters from to nicht richtig gesetzt", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;579;What if the cast fails?", FP.FP);
        putDef(ret, "#2R;579;Cast without check.", FP.FP);
        putDef(ret, "#2R;579;there is no getModel method", FP.FP);
        putDef(ret, "#2R;579;type checking reuired", FP.FP);
        putDef(ret, "#2R;579;Why do we have to cast the return value of getModel()? You should override the getModel method and return an object of the correct type to prevent ClassCastExceptions.", FP.FP); //design issue
        putDef(ret, "#2R;579;getModel() is probably a method of VFSDirectoryEntryTableModel? th", FP.FP);
        putDef(ret, "#2R;579;possible cast exception not handled", FP.FP); //design issue
        putDef(ret, "#2R;579;It would be nicer to have VFSDirectorEntryTableModel implement the TableColumnModelListener directly", FP.FP); //design issue
        putDef(ret, "#2R;579;Make this one a proper method in another class. This cast looks just strange.", FP.FP); //design issue
        putDef(ret, "#2R;579;from and to are swapped", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;579;From and to is swapped", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;579;to and from are interchanged in method call", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;579;to and from switched", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;579;Can't \"getModel\" return the correct type already? Or create a new method that does this", FP.FP); //design issue
        putDef(ret, "#2R;580;From and To Index vertauscht bei den Parametern", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;wrong order of \"from\" and \"to\"", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;The parameters order is switched. Try to turn on the proper securitu in the IDE.", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;param mismatch", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;\"from\" and \"to\" parameters swapped", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;fromIndex and toIndex used in wrong order", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;are indices from 0 to size - 1 or from 1 to size? potential error", WU.OFF_BY_ONE1);
        putDef(ret, "#2R;580;to und from vertauscht", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;wrong order of attributes", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Change order of parameters wrt. e.getToIndex() and e.getFromIndex()", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;switch parameters", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Wrong argument order: e.getFromIndex should be the first argument", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Bug: e.getTOIndex() wird in der Methode als int from genutzt und vice versa. Man sollte die Parameter vertauschen.", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Why are from/to swapped here?", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Wrong order of arguments, first from and second to", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;e.getToIndex() and e.getFromIndex() are misplaced and should be switched", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Bug: e.getTOIndex() is put into from and getFrom is put into to. Should be vice versa", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;arguments in the wrong order: first from, the to", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Die Reihenfolge der Parameter ist vertauscht", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;these parameters should be swapped", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Parameter vertauscht", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;ToIndex und FromIndex sind hier vertauscht", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;to/from indices are mixed up", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;The method actually takes `from` at first argument and `to` as second. In the call, here, those arguments are misplaced: `to` is first argument and `from` the second one.", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;The first index should be the from-index and the second should be the to-index, i.e. switch the parameters.", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;To and From indices may be inverted", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;wrong parametrization: from and to are swapped", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Prüfen, ob Index 0-basiert ist. Das würde in der Methode columnMoved zu weiteren Problemen führen!", WU.OFF_BY_ONE2);
        putDef(ret, "#2R;580;from und to ist vertauscht", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;parameters used in wrong order", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;From/to reversed", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Input parameters \"from\" and \"to\" are inverted", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;switch from/to paramter", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;From and to-index are changed. The method expects from as first parameter and to as second parameter, but you provide to first and from as second parameter. This may not work.", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;to and from are swapped", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Method columnMoved expects from followed by to as parameters, but given is to followed by from (wrong order of parametersI", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;parameters in wrong order. Should be first from then to", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;to and from are in wrong order", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;inverted arguments!", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;From and to have to be swapped and are the indexes in TableColumnModelEvent zero based?", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;es soll umgekehrt sein: zuerst e.getFromIndex und dann e.getToIndex", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Wrong argument order!!", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;API takes 'from' then the 'to' argument. here we pass them in reverse which will result in wrong behavior", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;check if parameter order is switched. defined as from/to, used as to/from", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;to and from reverted", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;from and to switched", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;arguments have to be swapped (first \"from\", than \"to\"), according to function definition in line 275", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;`from` and `to` got switched around", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;check the order of paramters, from and to", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;From / to don't match receiver", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Switch getToIndex() and getFromIndex()", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;swap parameters from / to: from is first, to is second", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;from <-> to", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Looks like you need to swap arguments here, see VFSDirectoryEntryTableModel.java#275", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;You should probably swap these parameters: from is the first one and to is the second one in the function definition", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Aren't those index inverted? columnMoved methods signature first parameter is from, second one is to", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;Indices from and to are mixed. from should be first, and to should be second.", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;The method actually takes `from` at first argument and `to` as second. (See org/experiment/editor/browser/VFSDirectoryEntryTableModel.java:275). In the call performed here, those arguments are misplaced: `to` is the first argument and `from` the second one.", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;580;to and from appear to be interchanged. this will fail in the column moved.", WU.FROM_TO_SWAP);
        putDef(ret, "#2R;582;why dont't you do anything in this method?", FP.FP);
        putDef(ret, "#2R;579;unchecked cast", FP.FP);
        putDef(ret, "#1R;283;this does not work if \"to > from\", as the \"to\" index is not decremented in this case after removing the element at \"from - 1\"", FP.FP);
        return ret;
    }
}