package de.unihannover.orderingExperimentAnalysis;

import static org.junit.Assert.assertEquals;

import java.io.StringReader;

import org.junit.Test;

import de.unihannover.orderingExperimentAnalysis.Tokenizer;

public class TokenizerTest {

    private static Tokenizer create(String string) {
        return new Tokenizer(new StringReader(string));
    }

    @Test
    public void testSimpleExpression() throws Exception {
        final Tokenizer t = create("12+abc");
        assertEquals("12", t.next());
        assertEquals("+", t.next());
        assertEquals("abc", t.next());
        assertEquals(null, t.next());
    }

    @Test
    public void testSimpleExpressionWithWhitespace() throws Exception {
        final Tokenizer t = create("   12  +  \n abc  / __var   ");
        assertEquals("12", t.next());
        assertEquals("+", t.next());
        assertEquals("abc", t.next());
        assertEquals("/", t.next());
        assertEquals("__var", t.next());
        assertEquals(null, t.next());
    }

    @Test
    public void testSomeStrings() throws Exception {
        final Tokenizer t = create("  \"s\" + \"123 456\" + \"\\\\\" + \"asd\\\"fgh\" ");
        assertEquals("\"s\"", t.next());
        assertEquals("+", t.next());
        assertEquals("\"123 456\"", t.next());
        assertEquals("+", t.next());
        assertEquals("\"\\\\\"", t.next());
        assertEquals("+", t.next());
        assertEquals("\"asd\\\"fgh\"", t.next());
        assertEquals(null, t.next());
    }

    @Test
    public void testSomeChars() throws Exception {
        final Tokenizer t = create("  's' + '1' + '\"' + '\\n' + '\\'' ");
        assertEquals("'s'", t.next());
        assertEquals("+", t.next());
        assertEquals("'1'", t.next());
        assertEquals("+", t.next());
        assertEquals("'\"'", t.next());
        assertEquals("+", t.next());
        assertEquals("'\\n'", t.next());
        assertEquals("+", t.next());
        assertEquals("'\\''", t.next());
        assertEquals(null, t.next());
    }

    @Test
    public void testAnnotation() throws Exception {
        final Tokenizer t = create("  @Override\n  public void xyz() {");
        assertEquals("@Override", t.next());
        assertEquals("public", t.next());
        assertEquals("void", t.next());
        assertEquals("xyz", t.next());
        assertEquals("(", t.next());
        assertEquals(")", t.next());
        assertEquals("{", t.next());
        assertEquals(null, t.next());
    }

    @Test
    public void testLineComment() throws Exception {
        final Tokenizer t = create("  asdf//these are no tokens\nbut this");
        assertEquals("asdf", t.next());
        assertEquals("but", t.next());
        assertEquals("this", t.next());
        assertEquals(null, t.next());
    }

    @Test
    public void testBlockComment() throws Exception {
        final Tokenizer t = create("  asdf/**these/are\n no * tokens*/but this");
        assertEquals("asdf", t.next());
        assertEquals("but", t.next());
        assertEquals("this", t.next());
        assertEquals(null, t.next());
    }
}
